Package: tabstats
Type: Package
Title: A Lightweight Toolkit for Displaying Customizable Tables
Version: 0.2.1
Authors@R: c(
    person(
        "Joshua", "Marie",
        email = "joshua.marie.k@gmail.com",
        role = c("aut", "cre")
    )
    )
Maintainer: Joshua Marie <joshua.marie.k@gmail.com>
Description: A lightweight toolkit that provides functions for printing tables from
    input data in the R console or terminal with customizable formatting. Supported
    outputs include American Psychological Association (APA)-style tables
    (American Psychological Association, 2020, ISBN:9781433832178),
    correlation matrices, contingency tables, and two-column summary tables.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.1.0)
URL: https://github.com/s7-stats/tabstats
BugReports: https://github.com/s7-stats/tabstats/issues
Imports: cli, tibble, vctrs, utils, stats
Suggests: broom, rstatix, glue, tidyr, testthat (>= 3.0.0), knitr,
        rmarkdown, fansi, crayon, dplyr
Config/testthat/edition: 3
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-07-08 10:21:52 UTC; DESKTOP
Author: Joshua Marie [aut, cre]
Repository: CRAN
Date/Publication: 2026-07-08 10:50:02 UTC
Built: R 4.4.3; ; 2026-07-08 15:13:41 UTC; unix

Package: tabstats
Type: Package
Title: A Lightweight Toolkit for Displaying Customizable Tables
Version: 0.2.0
Authors@R: c(
    person(
        "Joshua", "Marie",
        email = "joshua.marie.k@gmail.com",
        role = c("aut", "cre")
    )
    )
Maintainer: Joshua Marie <joshua.marie.k@gmail.com>
Description: A lightweight toolkit that provides functions for printing tables from
    input data in the R console or terminal with customizable formatting. Supported
    outputs include American Psychological Association (APA)-style tables
    (American Psychological Association, 2020, ISBN:9781433832178),
    correlation matrices, contingency tables, and two-column summary tables.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.1.0)
URL: https://github.com/joshuamarie/tabstats
BugReports: https://github.com/joshuamarie/tabstats/issues
Imports: cli, tibble, vctrs, utils, stats
Suggests: broom, rstatix, glue, tidyr, testthat (>= 3.0.0), knitr,
        rmarkdown, fansi, crayon, dplyr
Config/testthat/edition: 3
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-05-25 11:20:53 UTC; DESKTOP
Author: Joshua Marie [aut, cre]
Repository: CRAN
Date/Publication: 2026-05-25 11:40:02 UTC
Built: R 4.6.0; ; 2026-05-25 13:04:52 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(format,tabstats_style)
S3method(print,tabstats_style)
export(cm_style)
export(corr_matrix)
export(cross_table)
export(ct_style)
export(new_corr_data)
export(new_pairwise_data)
export(pairwise_matrix)
export(sm_style)
export(table_default)
export(table_summary)
export(tabstats_options)
export(td_style)

# tabstats <img src="man/figures/logo.png" align="right" alt="" width="120"/>

[![CRAN
status](https://www.r-pkg.org/badges/version/tabstats)](https://CRAN.R-project.org/package=tabstats)
[![R-CMD-check](https://github.com/joshuamarie/tabstats/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/joshuamarie/tabstats/actions/workflows/R-CMD-check.yaml)

## Package overview

Title: ***A Lightweight Toolkit for Displaying Customizable Tables***

***tabstats*** is a lightweight package in action, ideal for
development, that allows end users to print out the data they want to
present as a table in the command line, with further enhancements that
modify the table with styles and alignment. This is very useful when
displaying statistical results on the R console/command line as a table.

## Installation

Install the following package from CRAN:

``` r
install.packages("tabstats")
```

Or install the development version from GitHub:

``` r
# install.packages("pak")
pak::pak("joshuamarie/tabstats")
## devtools::install_github("joshuamarie/tabstats") 
```

## Example

Here’s a basic example that demonstrates how to use `tabstats` to format
and style a table:

``` r
table_default(head(mtcars, 5))
#> ────────────────────────────────────────────────────────────────────
#>    mpg    cyl  disp  hp   drat    wt     qsec   vs  am  gear  carb  
#> ────────────────────────────────────────────────────────────────────
#>   21.000   6   160   110  3.900  2.620  16.460  0   1    4     4    
#>   21.000   6   160   110  3.900  2.875  17.020  0   1    4     4    
#>   22.800   4   108   93   3.850  2.320  18.610  1   1    4     1    
#>   21.400   6   258   110  3.080  3.215  19.440  1   0    3     1    
#>   18.700   8   360   175  3.150  3.440  17.020  0   0    3     2    
#> ────────────────────────────────────────────────────────────────────
```

This package allows end users to apply advanced styling and alignment to
tables, such as color coding, column alignment, and borders.

## License

MIT + file LICENSE

## Code of Conduct

Please note that the `tabstats` project is released with a [Contributor
Code of
Conduct](https://contributor-covenant.org/version/2/1/CODE_OF_CONDUCT.html).
By contributing to this project, you agree to abide by its terms.

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
    collapse = FALSE,
    comment = NULL
)

options(cli.unicode = FALSE)
options(crayon.enabled = TRUE)

ansi_aware_handler = function(x, options) {
    paste0(
        "<pre class=\"r-output\"><code>",
        fansi::sgr_to_html(x = x, warn = FALSE, term.cap = "256"),
        "</code></pre>"
    )
}

knitr::knit_hooks$set(
    output = ansi_aware_handler, 
    message = ansi_aware_handler, 
    warning = ansi_aware_handler,
    error = ansi_aware_handler
)

knitr::opts_chunk$set(
    collapse = TRUE,
    # comment = "#>",
    comment = NA, 
    # fig.path = "man/figures/README-",
    out.width = "100%"
)

library(tabstats)

## ----eval = FALSE-------------------------------------------------------------
# install.packages("tabstats")

## ----eval = FALSE-------------------------------------------------------------
# # install.packages("pak")
# pak::pak("joshuamarie/tabstats")
# ## devtools::install_github("joshuamarie/tabstats")

## -----------------------------------------------------------------------------
head(mtcars[, 1:5], 5)

## -----------------------------------------------------------------------------
table_default(head(mtcars[, 1:5], 5))

## -----------------------------------------------------------------------------
df = data.frame(
    Statistic = c("N", "Mean", "SD", "Min", "Max"),
    Value = c("100", "3.45", "1.20", "1.00", "6.00")
)

table_summary(
    df, 
    title = "Descriptive Statistics", 
    header = TRUE
)

## -----------------------------------------------------------------------------
corr_matrix(cor(mtcars[, 1:4]), method = "Pearson")

## -----------------------------------------------------------------------------
cor_mat = 
    iris |> 
    rstatix::cor_test(Sepal.Width, Sepal.Length, Petal.Length) |> 
    dplyr::mutate(
        var1,
        var2,
        cor = format(cor, digits = 2),
        statistic = format(statistic, digits = 2),
        conf_int = paste0(
            "[",
            format(conf.low, digits = 2), 
            ", ",
            format(conf.high, digits = 2),
            "]"
        ),
        
        .keep = "unused"
    ) 

cor_mat |> 
    with({
        corr_matrix(
            new_corr_data(
                var1 = var1,
                var2 = var2,
                corr = cor,
                statistic = statistic,
                pval = p,
                conf_int = conf_int
            ),
            title = "Pearson Correlation Matrix"
        )
    })

## -----------------------------------------------------------------------------
m = matrix(
    c(10, 20, 30, 40), 
    nrow = 2,
    dimnames = list(
        c("A", "B"), 
        c("X", "Y")
    )
)

cross_table(m, percentage = "all")

## -----------------------------------------------------------------------------
table_summary(
    df, 
    title = "Descriptive Statistics", 
    header = TRUE, 
    style = sm_style(
        left_col = "blue_bold",
        right_col = "green",
        title = "bold", 
        sep = ":  "
    )
)

## -----------------------------------------------------------------------------
table_summary(
    df, 
    title = "Descriptive Statistics", 
    header = TRUE, 
    style = sm_style(
        left_col = \(x) cli::col_red(cli::style_bold(x)),
        right_col = \(x) cli::col_cyan(x), 
        title = "bold", 
        sep = ":  "
    )
)

## -----------------------------------------------------------------------------
table_summary(
    df, 
    title = "Descriptive Statistics", 
    header = TRUE, 
    style = sm_style(
        left_col = function(x) {
            red = cli::make_ansi_style("#BD4444")
            red(cli::style_bold(x))
        },
        right_col = \(x) {
            bluish_color = cli::make_ansi_style("#67A2C5")
            bluish_color(x)
        }, 
        title = "bold", 
        sep = ":  "
    )
)

## -----------------------------------------------------------------------------
cor_mat |> 
    with({
        corr_matrix(
            new_corr_data(
                var1 = var1,
                var2 = var2,
                corr = cor,
                statistic = statistic,
                pval = p,
                conf_int = conf_int
            ),
            title = "Pearson Correlation Matrix",
            style = cm_style(
                pval = function(x) {
                    x_num = as.numeric(x)
                    if (is.na(x_num) || x_num > 0.05) {
                        cli::style_italic(x) 
                    } else if (x_num > 0.01) {
                        reddish = cli::make_ansi_style("#CE2626")
                        reddish(x)
                        # or just cli::col_red(x)
                    } else {
                        cli::style_bold("<0.001")
                    }
                }
            )
        )
    })

.onLoad = function(libname, pkgname) {
    options(
        tab_default = list(
            vb_char = "\u2502",           # '│
            vb_top = "\u252C",            # '┬'
            vb_mid = "\u253C",            # '┼'
            vb_bottom = "\u2534",         # '┴'
            border_char = "\u2500",       # '─'
            header_underline = "\u2500",  # '─'
            truncate_message = TRUE,
            nrows = 10
        ),
        tab_digits = list(
            ex = 1,
            row_pct = 0,
            col_pct = 0,
            total_pct = 0
        )

        # ... Add more options (leave blank)
    )

    # ---- Cli themes ----
    cli::cli_div(theme = list(
        span.info = list(color = "cyan"),
        span.warning = list(color = "yellow")
    ))
}

#' Manage package options
#'
#' This function allows retrieving or modifying the package options across different categories.
#' If called without arguments, it returns all option categories and their values.
#' If `category` is provided alone, it returns all options in that category.
#' If `category` and `option` are provided, it returns the specific option value.
#' If all three parameters are provided, it updates the specified option.
#'
#' @param category A character string specifying the option category (e.g., "tab_default", "tab_digits").
#'                 If omitted, returns all option categories.
#' @param option A character string specifying the option to retrieve or modify within the category.
#'               For backward compatibility, you can also use a specific option name directly as the `category` parameter.
#' @param value The new value to assign to the specified option. If NULL, the function returns the current value.
#' @return If no arguments are provided, returns all option categories and their values.
#'         If only `category` is provided, returns all options in that category.
#'         If `category` and `option` are provided without `value`, returns the current value of that option.
#'         If all parameters are provided, updates the option and returns the updated option category list invisibly.
#' @examples
#' # Get all options across all categories
#' tabstats_options()
#'
#' # Get all options in the "tab_default" category
#' tabstats_options("tab_default")
#'
#' # Get all options in the "tab_digits" category
#' tabstats_options("tab_digits")
#'
#' # Get a specific option
#' tabstats_options("tab_default", "vb_top")
#' tabstats_options("tab_digits", "ex")
#'
#' # Using backward compatibility (system will find the right category)
#' tabstats_options("vb_top")
#' tabstats_options("ex")
#'
#' # Modify an option
#' tabstats_options("tab_default", "border_char", "+")
#' tabstats_options("tab_digits", "ex", 2)
#'
#' # Using backward compatibility for modification
#' tabstats_options("border_char", "+")
#' tabstats_options("ex", 2)
#'
#' @export
tabstats_options = function(category = NULL, option = NULL, value = NULL) {
    # --- Add more categories... ----
    valid_categories = c("tab_default", "tab_digits")


    if (is.null(category)) {
        result = list()
        for (cat in valid_categories) {
            result[[cat]] = getOption(cat, default = list())
        }
        return(result)
    }

    if (!category %in% valid_categories && !category %in% unlist(lapply(valid_categories, function(cat) {
        names(getOption(cat, default = list()))
    }))) {
        stop("Invalid category or option name: ", category)
    }

    is_option = FALSE
    option_category = NULL

    for (cat in valid_categories) {
        cat_options = names(getOption(cat, default = list()))
        if (category %in% cat_options) {
            is_option = TRUE
            option_category = cat
            break
        }
    }

    if (is_option) {
        actual_option = category
        actual_category = option_category
        actual_value = option

        cat_opts = getOption(actual_category, default = list())

        if (is.null(actual_value)) {
            return(cat_opts[[actual_option]])
        }

        cat_opts[[actual_option]] = actual_value
        options(structure(list(cat_opts), names = actual_category))
        return(invisible(cat_opts))
    }

    if (is.null(option)) {
        return(getOption(category, default = list()))
    }

    cat_opts = getOption(category, default = list())

    if (!option %in% names(cat_opts)) {
        stop("Option '", option, "' not found in category '", category, "'")
    }

    if (is.null(value)) {
        return(cat_opts[[option]])
    }

    cat_opts[[option]] = value
    options(structure(list(cat_opts), names = category))
    invisible(cat_opts)
}

test_that("tabstats_options() with no args returns a named list with all categories", {
    opts = tabstats_options()
    expect_type(opts, "list")
    expect_true("tab_default" %in% names(opts))
    expect_true("tab_digits" %in% names(opts))
})

test_that("tabstats_options('tab_default') returns the tab_default list", {
    td = tabstats_options("tab_default")
    expect_type(td, "list")
    expect_true("border_char" %in% names(td))
    expect_true("nrows" %in% names(td))
})

test_that("tabstats_options('tab_digits') returns the tab_digits list", {
    tdig = tabstats_options("tab_digits")
    expect_type(tdig, "list")
    expect_true("ex" %in% names(tdig))
})

test_that("tabstats_options retrieves a specific option by category + name", {
    val = tabstats_options("tab_default", "nrows")
    expect_type(val, "double")
    expect_equal(val, 10)
})

test_that("tabstats_options backward-compat: option name used as category", {
    val = tabstats_options("nrows")
    expect_equal(val, 10)
})

test_that("tabstats_options sets an option and retrieves updated value", {
    original = tabstats_options("tab_default", "nrows")
    on.exit(tabstats_options("tab_default", "nrows", original))

    tabstats_options("tab_default", "nrows", 25)
    expect_equal(tabstats_options("tab_default", "nrows"), 25)
})

test_that("tabstats_options backward-compat set: option name used as category", {
    original = tabstats_options("nrows")
    on.exit(tabstats_options("nrows", original))

    tabstats_options("nrows", 5)
    expect_equal(tabstats_options("nrows"), 5)
})

test_that("tabstats_options errors on invalid category", {
    expect_error(tabstats_options("nonexistent_category"), "Invalid category")
})

test_that("tabstats_options errors on unknown option within a valid category", {
    expect_error(
        tabstats_options("tab_default", "not_an_option"),
        "not found in category"
    )
})

center_text_x2 = function(text, width) {
    text_width = nchar(strip_ansi(text))
    padding = max(0, width - text_width)
    left_padding = floor(padding / 2)
    right_padding = ceiling(padding / 2)
    paste0(strrep(" ", left_padding), text, strrep(" ", right_padding))
}

format_number_x2 = function(x) {
    vctrs::vec_if_else(
        abs(x - round(x)) < 1e-10,
        sprintf("%d", round(x)),
        sprintf("%.1f", x)
    )
}

strip_ansi = function(text) {
    gsub("\033\\[[0-9;]*m", "", text)
}

#' Display a Pairwise Matrix Table in the Console
#'
#' @param display A `pairwise_spec` object from `new_pairwise_data()`, or a
#'   plain symmetric matrix (e.g. from `cor()`).
#' @param title Label shown above the table. Auto-detected from a `title`
#'   attribute on the spec if present; falls back to `name`.
#' @param name Short name used in the layout legend box header. Default
#'   `"Pairwise Matrix"`.
#' @param diag_1 If `TRUE`, diagonal cells always show `"1"`. Default `TRUE`.
#' @param digits Decimal places for numeric formatting. Default `3`.
#' @param layout_view Show a layout legend box above the table? Default `FALSE`.
#' @param layout_center Center the layout box in the terminal? Default `FALSE`.
#' @param center_table Center table in terminal? Default `FALSE`.
#' @param border_char Border character. Default from `getOption("tab_default")`.
#' @param style A style list. Keys match the extra field names passed to
#'   `new_pairwise_data()` (e.g. `rho`, `pval`, `bf`), plus `title` and
#'   `border_text`.
#' @param ... Reserved for future use.
#'
#' @return Invisibly returns the rendered character matrix.
#'
#' @examples
#' # From a plain correlation matrix
#' pairwise_matrix(cor(mtcars[, 1:4]), title = "Pearson Correlation Matrix")
#'
#' # Customizable example
#' spec = new_pairwise_data(
#'     var1 = c("a", "a", "b"),
#'     var2 = c("b", "c", "c"),
#'     rho = c("0.89", "0.79", "0.66"),
#'     pval = c("<0.001", "<0.001", "<0.001")
#' )
#' pairwise_matrix(spec, title = "Pearson Correlation Matrix", layout_view = TRUE)
#'
#' @export
pairwise_matrix = function(
        display,
        title = NULL,
        name = "Pairwise Matrix",
        diag_1 = TRUE,
        digits = 3,
        layout_view = FALSE,
        layout_center = FALSE,
        center_table = FALSE,
        border_char = getOption("tab_default")$border_char,
        style = list(),
        ...
) {
    if (!requireNamespace("cli", quietly = TRUE))
        stop("The 'cli' package is required. Please install it.")

    # ---- Validate style ----

    if (!is.null(style) && !is.list(style))
        cli::cli_abort("{.arg style} must be a list or a style object.")

    if (inherits(style, "tabstats_style") && !inherits(style, "cm_style")) {
        cli::cli_abort(
            "{.arg style} must be a {.cls cm_style} object for {.fn pairwise_matrix}.",
            "x" = "Got {.cls {class(style)[1]}}."
        )
    }

    # ---- Coerce input ----
    spec = if (is.matrix(display)) {
        matrix_spec_resolver_pm(display, digits, name)
    } else if (inherits(display, "pairwise_spec")) {
        display
    } else {
        stop("`display` must be a `pairwise_spec` object or a symmetric matrix.", call. = FALSE)
    }

    title = if (is.null(title)) {
        t = attr(spec, "title")
        if (is.null(t)) name else t
    } else {
        title
    }

    style = style_resolver_pm(style, names(spec$extras))

    field_names = names(spec$extras)
    if (length(field_names) == 0) field_names = "corr"
    n_fields = length(field_names)

    # ---- Layout view ----
    if (layout_view)
        print_layout_pm(field_names, style, border_char, layout_center, center_table, name)

    # ---- Build render matrix ----
    vars = spec$vars
    n = length(vars)

    mat = matrix("", nrow = n * n_fields, ncol = n + 1L)

    for (i in seq_len(n))
        mat[(i - 1L) * n_fields + 1L, 1L] = vars[i]

    for (p in seq_along(spec$var1)) {
        v1 = spec$var1[p]
        v2 = spec$var2[p]

        if (v1 == v2) {
            i = match(v1, vars)
            row_base = (i - 1L) * n_fields + 1L
            if (diag_1) {
                mat[row_base, i + 1L] = "1"
            } else {
                mat[row_base, i + 1L] = spec$extras[[1L]][p]
                for (k in seq_along(spec$extras)[-1L])
                    mat[row_base + k - 1L, i + 1L] = spec$extras[[k]][p]
            }
            next
        }

        ri = match(v1, vars)
        ci = match(v2, vars)

        if (spec$pattern %in% c("lt", "lte")) {
            if (ri < ci) { tmp = ri; ri = ci; ci = tmp }
        } else if (spec$pattern %in% c("gt", "gte")) {
            if (ri > ci) { tmp = ri; ri = ci; ci = tmp }
        }

        row_base = (ri - 1L) * n_fields + 1L
        mat[row_base, ci + 1L] = spec$extras[[1L]][p]

        for (k in seq_along(spec$extras)[-1L])
            mat[row_base + k - 1L, ci + 1L] = spec$extras[[k]][p]
    }

    # ---- Column widths ----
    col_names = c("Variable", vars)
    strip = function(x) nchar(gsub("\033\\[[0-9;]*m", "", x))

    col_widths = vapply(
        seq_len(ncol(mat)),
        function(j) {
            max(
                strip(col_names[j]),
                vapply(seq_len(nrow(mat)), function(i) strip(mat[i, j]), integer(1))
            )
        },
        integer(1)
    )

    total_width = sum(col_widths) + 3L * (length(col_widths) - 1L) + 4L
    hline = style[["border_text"]](strrep(border_char, total_width))
    prefix = left_pad_pm(center_table, total_width)

    # ---- Apply styles ----
    styled = matrix_styler_pm(mat, field_names, style)

    # ---- Print ----
    cat("\n", prefix, style[["title"]](align_center(title, total_width)), "\n", sep = "")
    cat(prefix, hline, "\n", sep = "")
    cat(prefix, format_row_pm(col_names, col_widths, left_align_first = TRUE), "\n", sep = "")
    cat(prefix, hline, "\n", sep = "")

    for (i in seq_len(nrow(mat))) {
        cat(prefix, format_row_pm(styled[i, ], col_widths, left_align_first = TRUE), "\n", sep = "")
        if (i %% n_fields == 0L)
            cat(prefix, hline, "\n", sep = "")
    }

    invisible(mat)
}

#' @rdname pairwise_matrix
#' @export
corr_matrix = function(
        display,
        title = NULL,
        diag_1 = TRUE,
        digits = 3,
        layout_view = FALSE,
        layout_center = FALSE,
        center_table = FALSE,
        border_char = getOption("tab_default")$border_char,
        style = list(),
        ...
) {
    if (inherits(display, "corr_spec") && !inherits(display, "pairwise_spec"))
        class(display) = c("corr_spec", "pairwise_spec")

    if (is.null(title)) {
        t = attr(display, "title")
        title = if (is.null(t)) "Correlation Matrix" else t
    }

    pairwise_matrix(
        display = display,
        title = title,
        name = "Corr. Matrix",
        diag_1 = diag_1,
        digits = digits,
        layout_view = layout_view,
        layout_center = layout_center,
        center_table = center_table,
        border_char = border_char,
        style = style,
        ...
    )
}

spec_bf = new_pairwise_data(
    var1 = c("a", "a", "b"),
    var2 = c("b", "c", "c"),
    bf = c("12.4", "3.1", "0.8")
)

spec_effect = new_pairwise_data(
    var1 = c("a", "a", "b"),
    var2 = c("b", "c", "c"),
    d = c("0.82", "0.45", "0.31"),
    ci = c("[0.61, 1.03]", "[0.24, 0.66]", "[0.10, 0.52]")
)

spec_icc = new_pairwise_data(
    var1 = c("rater1", "rater1", "rater2"),
    var2 = c("rater2", "rater3", "rater3"),
    icc = c("0.91", "0.87", "0.89"),
    ci = c("[0.85, 0.95]", "[0.79, 0.92]", "[0.82, 0.94]")
)

test_that("errors on non-matrix, non-pairwise_spec input", {
    expect_error(pairwise_matrix(list(a = 1)), "pairwise_spec.*symmetric matrix")
})

test_that("errors on non-symmetric matrix", {
    expect_error(pairwise_matrix(matrix(1:4, 2, 2)), "symmetric")
})

# ---- new_pairwise_data() ----

test_that("returns a pairwise_spec object", {
    expect_s3_class(spec_bf, "pairwise_spec")
})

test_that("errors when var1 and var2 have different lengths", {
    expect_error(
        new_pairwise_data(var1 = c("a", "b"), var2 = "c", bf = c("1.2", "3.4")),
        "same length"
    )
})

test_that("errors when extra vector has wrong length", {
    expect_error(
        new_pairwise_data(var1 = c("a"), var2 = c("b"), bf = c("1.2", "3.4")),
        "same length"
    )
})

test_that("accepts multiple extra fields", {
    expect_no_error(
        new_pairwise_data(
            var1 = c("a", "a", "b"),
            var2 = c("b", "c", "c"),
            d = c("0.82", "0.45", "0.31"),
            ci = c("[0.61, 1.03]", "[0.24, 0.66]", "[0.10, 0.52]")
        )
    )
})

# ---- pairwise_spec input ----

test_that("accepts a pairwise_spec and returns a matrix invisibly", {
    out = expect_invisible(pairwise_matrix(spec_bf, title = "Bayes Factor Matrix"))
    expect_true(is.matrix(out))
    expect_type(out, "character")
})

test_that("accepts a multi-field pairwise_spec (effect size + CI)", {
    expect_no_error(pairwise_matrix(spec_effect, title = "Cohen's d Matrix"))
})

test_that("accepts a multi-field pairwise_spec (ICC + CI)", {
    expect_no_error(pairwise_matrix(spec_icc, title = "ICC Matrix"))
})

test_that("name appears in layout legend header", {
    out = capture.output(
        pairwise_matrix(spec_bf, name = "BF Matrix", layout_view = TRUE)
    )
    expect_true(any(grepl("BF Matrix", out)))
})

test_that("name is used as fallback title when title is not supplied", {
    out = capture.output(pairwise_matrix(spec_bf, name = "Bayes Factor Matrix"))
    expect_true(any(grepl("Bayes Factor Matrix", out)))
})

test_that("accepts a symmetric matrix and returns invisibly", {
    m = cor(mtcars[, 1:4])
    out = expect_invisible(pairwise_matrix(m, title = "Pearson Correlation Matrix"))
    expect_true(is.matrix(out))
})

test_that("diagonal cells show '1' when diag_1 = TRUE (default)", {
    out = pairwise_matrix(cor(mtcars[, 1:3]))
    expect_true(any(out == "1"))
})

test_that("diagonal cells do not show '1' when diag_1 = FALSE", {
    m = cor(mtcars[, 1:3])
    out = pairwise_matrix(m, diag_1 = FALSE)
    diag_positions = seq(1, nrow(out) * ncol(out), by = ncol(out) + 1)
    expect_false(all(out[diag_positions] == "1"))
})

test_that("layout_view = TRUE prints without error", {
    expect_no_error(pairwise_matrix(spec_effect, layout_view = TRUE))
})

test_that("cm_style() works with pairwise_matrix", {
    expect_no_error(
        pairwise_matrix(spec_effect, style = cm_style(d = "blue_bold", ci = "red"))
    )
})

#' Display a formatted table in the console
#'
#' @param x A data frame or tibble.
#' @param justify_cols Alignment: a single string, vector, or named list of "left"/"right"/"center".
#' @param digits Digits to round numeric columns to. Default `3`.
#' @param digits_by_col Named list of per-column digit overrides.
#' @param scientific Display numerics in scientific notation? Default `FALSE`.
#' @param na_print String for missing values. Default `""`.
#' @param min_width Minimum column width. Default `NULL`.
#' @param border_char Character for borders. Default `"\u2500"`.
#' @param show_row_names Show row names? Default `FALSE`.
#' @param center_table Center table in terminal? Default `FALSE`.
#' @param n_space Spaces between columns. Default `2`.
#' @param title Optional title string above the table.
#'   from [td_style()], or a named list where each name is a column name or
#'   `"title"`, and each value is either a cli style string (e.g. `"blue_bold"`)
#'   or a function `\(ctx) ...` receiving a context list.
#' @param style_colnames Styling for column header cells. A `td_style` object
#'   from [td_style()], or a named list where each name is a column name or
#'   `"title"`, and each value is either a cli style string (e.g. `"blue_bold"`)
#'   or a function `\(ctx) ...` receiving a context list.
#' @param style_columns Styling for data cells. A `td_style` object from
#'   [td_style()], or a named list where each name is a column name or column
#'   index as a string, and each value is a cli style string or a function
#'   `\(ctx) ...` receiving a context list with elements `value`,
#'   `formatted_value`, `col_name`, `col_index`, `is_header`, `data`,
#'   `justify`, and `width`.
#' @param nrows Max rows to display before truncation.
#' @param vb Vertical border spec: `list(char = "\u2502", after = c(1, 3))`.
#' @param auto_wrap Auto-wrap wide tables? Default `TRUE`.
#' @param wrap_threshold Fraction of console width before wrapping. Default `1`.
#' @param ... Reserved for future use.
#'
#' @return Invisibly returns the input data as a character matrix after
#'   formatting has been applied. The function is called primarily for
#'   its side effect of printing a styled table to the R console.
#'   Returns `invisible(NULL)` early if the input has 0 rows and 0 columns,
#'   or if it has 0 columns.
#'
#' @examples
#' table_default(head(mtcars))
#' table_default(head(mtcars), style_columns = td_style(mpg = "cyan", cyl = "magenta"))
#'
#' @export
table_default = function(
    x,
    justify_cols = "center",
    digits = 3,
    digits_by_col = NULL,
    scientific = FALSE,
    na_print = "",
    min_width = NULL,
    border_char = options("tab_default")$tab_default$border_char,
    show_row_names = FALSE,
    center_table = FALSE,
    n_space = 2,
    title = NULL,
    style_colnames = NULL,
    style_columns = NULL,
    nrows = getOption("tab_default")$nrows,
    vb = list(),
    auto_wrap = TRUE,
    wrap_threshold = 1,
    ...
) {

    # ---- Coerce input --------------------------------------------------------
    if (!inherits(x, "data.frame")) {
        x = try(as.data.frame(x), silent = TRUE)
        if (inherits(x, "try-error"))
            stop("`x` must be a data frame or coercible to one.", call. = FALSE)
    }

    # ---- Validate style ----

    if (!is.null(style_colnames) && !inherits(style_colnames, "td_style")) {
        cli::cli_abort(
            "{.arg style_colnames} must be a {.cls td_style} object.",
            "i" = "Use {.fn td_style} to build one."
        )
    }

    if (!is.null(style_columns) && !inherits(style_columns, "td_style")) {
        cli::cli_abort(
            "{.arg style_columns} must be a {.cls td_style} object.",
            "i" = "Use {.fn td_style} to build one."
        )
    }

    # ---- Validate scalar params ----------------------------------------------
    if (!is.numeric(n_space) || n_space < 0) {
        cli::cli_warn("`n_space` must be non-negative; using default 2.")
        n_space = 2
    }
    if (!is.numeric(nrows) || nrows < 0) {
        cli::cli_warn("`nrows` must be non-negative; using default 10.")
        nrows = 10
    }
    n_space = floor(n_space)
    nrows = floor(nrows)

    # ---- Truncation ----------------------------------------------------------
    original_nrow = nrow(x)
    truncated = original_nrow > nrows
    if (truncated) x = utils::head(x, nrows)

    # ---- Row names -----------------------------------------------------------
    x = tibble::as_tibble(x, rownames = if (show_row_names) "row_names" else NA)
    if (show_row_names && !"row_names" %in% names(x)) {
        # x = dplyr::mutate(x, row_names = as.character(seq_len(nrow(x))), .before = 1)
        x$row_names = as.character(seq_len(nrow(x)))
        x = x[, c("row_names", setdiff(names(x), "row_names")), drop = FALSE]
    } else if (show_row_names) {
        # x = dplyr::relocate(x, "row_names", .before = 1)
        x = x[, c("row_names", setdiff(names(x), "row_names")), drop = FALSE]
    }

    original_x = x

    # ---- Early exits ---------------------------------------------------------
    if (nrow(x) == 0 && ncol(x) == 0) {
        cat("Empty data frame (0 rows, 0 columns)\n")
        return(invisible(NULL))
    }
    if (ncol(x) == 0) {
        cat("Data frame has 0 columns.\n")
        return(invisible(NULL))
    }

    # ---- Format & measure ----------------------------------------------------
    x = data_formatter(x, digits, digits_by_col, scientific, na_print)
    col_names = colnames(x)
    x_chars = as.matrix(x)
    col_widths = col_widths_total(x, x_chars, col_names, style_columns, original_x, min_width)

    # ---- Vertical border setup -----------------------------------------------
    vb_setup = vb_resolve(vb, col_names, col_widths)

    # ---- Terminal & wrap check -----------------------------------------------
    term_width = tryCatch(
        cli::console_width(),
        error = function(e) as.integer(getOption("width")) %||% 80
    )
    total_width = width_total(col_widths, n_space, vb_setup$n_borders, nchar(vb_setup$char))
    should_wrap = auto_wrap && total_width > (term_width * wrap_threshold) && length(col_widths) > 2

    # ---- Print ---------------------------------------------------------------
    if (truncated)
        cli::cli_alert_info("Showing {nrows} of {original_nrow} rows")

    if (should_wrap) {
        wrapped_table_print(
            x_chars, col_names, col_widths, vb_setup,
            justify_cols, n_space, title,
            style_colnames, style_columns, original_x,
            center_table, term_width, border_char, wrap_threshold
        )
    } else {
        default_table_print(
            x_chars, col_names, col_widths, vb_setup,
            justify_cols, n_space, title,
            style_colnames, style_columns, original_x,
            center_table, term_width, total_width, border_char
        )
    }

    if (truncated)
        cli::cli_alert_warning("Displayed only {nrows} of {original_nrow} rows. Set `nrows` to show more.")

    invisible(x)
}

#' Style specification for table_default()
#'
#' Constructs a validated style object for use with [table_default()].
#'
#' @param ... Named style entries. Each name must be a column name, a column
#'   index as a string (e.g. `"1"`), or `"title"`. Each value is a string or
#'   a function `\(ctx) ...` where `ctx` is a context list.
#'
#' @return An object of class `c("td_style", "tabstats_style")`.
#'
#' @examples
#' td_style(mpg = "cyan", cyl = "magenta")
#' td_style(mpg = \(ctx) cli::col_red(ctx$value), title = "bold")
#'
#' @export
td_style = function(...) {
    args = list(...)
    for (nm in names(args)) {
        val = args[[nm]]
        if (!is.character(val) && !is.function(val)) {
            cli::cli_abort(
                "Style entry {.arg {nm}} must be a string or a function."
            )
        }
    }

    vctrs::new_vctr(args, class = c("td_style", "tabstats_style"))
}

#' Style specification for table_summary()
#'
#' Constructs a validated style object for use with [table_summary()].
#'
#' @param left_col Style for the left column. A string (e.g. `"blue_bold"`)
#'   or a function `\(x) ...`.
#' @param right_col Style for the right column.
#' @param border_text Style for the horizontal border lines.
#' @param title Style for the title text.
#' @param sep A single character used as the column separator (e.g. `"|"`).
#'
#' @return An object of class `c("sm_style", "tabstats_style")`.
#'
#' @examples
#' sm_style(left_col = "blue_bold", right_col = "green", title = "bold")
#' sm_style(left_col = \(x) cli::col_red(x), sep = "|")
#'
#' @export
sm_style = function(left_col = NULL, right_col = NULL, border_text = NULL, title = NULL, sep = NULL) {
    args = Filter(
        Negate(is.null),
        list(
            left_col = left_col,
            right_col = right_col,
            border_text = border_text,
            title = title,
            sep = sep
        )
    )

    new_tabstats_style(
        args,
        valid_keys = c("left_col", "right_col", "border_text", "title", "sep"),
        class_name = "sm_style"
    )
}

#' Style specification for cross_table()
#'
#' Constructs a validated style object for use with [cross_table()].
#'
#' @param observed Style for observed frequency cells.
#' @param expected Style for expected frequency values.
#' @param row_percentage Style for row percentage values.
#' @param col_percentage Style for column percentage values.
#' @param total_percentage Style for grand total percentage values.
#' @param total Style for total row/column cells.
#' @param title Style for the table title.
#' @param border_text Style for the horizontal border lines.
#'
#' @return An object of class `c("ct_style", "tabstats_style")`.
#'
#' @examples
#' ct_style(observed = "blue", expected = "yellow", title = "bold")
#' ct_style(observed = \(ctx) cli::col_green(ctx$formatted_text))
#'
#' @export
ct_style =
    function(
        observed = NULL,
        expected = NULL,
        row_percentage = NULL,
        col_percentage = NULL,
        total_percentage = NULL,
        total = NULL,
        title = NULL,
        border_text = NULL
    ) {
        args = Filter(Negate(is.null), list(
            observed = observed,
            expected = expected,
            row_percentage = row_percentage,
            col_percentage = col_percentage,
            total_percentage = total_percentage,
            total = total,
            title = title,
            border_text = border_text
        ))

        new_tabstats_style(
            args,
            valid_keys = c(
                "observed",
                "expected",
                "row_percentage",
                "col_percentage",
                "total_percentage",
                "total",
                "title",
                "border_text"
            ),
            class_name = "ct_style"
        )
    }

#' Style specification for corr_matrix()
#'
#' Constructs a validated style object for use with [corr_matrix()].
#'
#' @param ... Named style entries. Names should match the extra field names
#'   passed to [new_corr_data()] (e.g. `rho`, `pval`, `bf`), or the reserved
#'   keys `title` and `border_text`.
#'
#' @return An object of class `c("cm_style", "tabstats_style")`.
#'
#' @examples
#' cm_style(rho = "blue_bold", pval = "red", title = "bold")
#' cm_style(rho = \(x) cli::col_cyan(x))
#'
#' @export
cm_style = function(...) {
    args = list(...)
    for (nm in names(args)) {
        val = args[[nm]]
        if (!is.character(val) && !is.function(val)) {
            cli::cli_abort(
                "Style entry {.arg {nm}} must be a string or a function."
            )
        }
    }

    vctrs::new_vctr(args, class = c("cm_style", "tabstats_style"))
}

new_tabstats_style = function(style_list, valid_keys, class_name) {
    unknown = setdiff(names(style_list), valid_keys)
    if (length(unknown) > 0) {
        cli::cli_abort(c(
            "Unknown style {cli::qty(unknown)} key{?s}: {.arg {unknown}}.",
            "i" = "Valid keys are: {.arg {valid_keys}}."
        ))
    }

    for (nm in names(style_list)) {
        val = style_list[[nm]]
        if (!is.character(val) && !is.function(val)) {
            cli::cli_abort(
                "Style key {.arg {nm}} must be a string or a function."
            )
        }
    }

    vctrs::new_vctr(
        style_list,
        valid_keys = valid_keys,
        class = c(class_name, "tabstats_style")
    )
}

#' @export
format.tabstats_style = function(x, ...) {
    cls = setdiff(class(x), "tabstats_style")[1]
    header = sprintf("<tabstats_style / %s>", cls)
    if (length(x) == 0)
        return(c(header, "  (no keys set)"))

    entries = vapply(
        names(x),
        function(nm) {
            val = x[[nm]]
            desc = if (is.function(val)) "<function>" else paste0('"', val, '"')
            sprintf("  %s: %s", nm, desc)
        },
        character(1)
    )

    c(header, entries)
}

#' @export
print.tabstats_style = function(x, ...) {
    cat(format(x, ...), sep = "\n")
    invisible(x)
}

df = data.frame(
    Category = c("A", "B", "C", "D", "E"),
    Value = c(10, 20, 30, 40, 50)
)

df_long = data.frame(
    Variable = c("N", "Mean", "SD", "Median", "Min", "Max", "Skew", "Kurt", "SE", "IQR"),
    Value = c("200", "4.12", "0.98", "4.00", "1.50", "6.50", "0.23", "2.87", "0.07", "1.25")
)

# ---- Input validation ----

test_that("rejects non-data-frame input", {
    expect_error(table_summary(matrix(1:4, 2, 2)), "data frame")
})

test_that("rejects data frame with more than 2 columns", {
    expect_error(table_summary(data.frame(a = 1, b = 2, c = 3)), "2 columns")
})

test_that("rejects data frame with fewer than 2 columns", {
    expect_error(table_summary(data.frame(a = 1)), "2 columns")
})

# ---- Basic output ----

test_that("prints without error with default args", {
    expect_no_error(table_summary(df))
})

test_that("returns NULL invisibly", {
    expect_invisible(table_summary(df))
    expect_null(table_summary(df))
})

test_that("output contains all left-column values", {
    out = capture.output(table_summary(df))
    for (v in c("A", "B", "C", "D", "E"))
        expect_true(any(grepl(v, out)))
})

test_that("output contains all right-column values", {
    out = capture.output(table_summary(df))
    for (v in c("10", "20", "30", "40", "50"))
        expect_true(any(grepl(v, out)))
})

test_that("output has exactly two border lines by default", {
    out = capture.output(table_summary(df))
    borders = sum(grepl("^-+$", trimws(out)))
    expect_equal(borders, 2L)
})

# ---- Title ----

test_that("title appears in output when provided", {
    out = capture.output(table_summary(df, title = "My Table"))
    expect_true(any(grepl("My Table", out)))
})

test_that("title is absent when title = NULL", {
    out = capture.output(table_summary(df, title = NULL))
    expect_false(any(grepl("My Table", out)))
})

test_that("title with style runs without error", {
    expect_no_error(
        table_summary(df, title = "Styled", style = sm_style(title = "bold"))
    )
})

# ---- Header ----

test_that("header = TRUE prints both column names", {
    out = capture.output(table_summary(df, header = TRUE))
    expect_true(any(grepl("Category", out)))
    expect_true(any(grepl("Value", out)))
})

test_that("header = TRUE adds a third border line", {
    out = capture.output(table_summary(df, header = TRUE))
    borders = sum(grepl("^-+$", trimws(out)))
    expect_equal(borders, 3L)
})

test_that("header = FALSE does not print column names", {
    out = capture.output(table_summary(df, header = FALSE))
    expect_false(any(grepl("Category", out)))
})

# ---- Split table (l) ----

test_that("split table prints without error", {
    expect_no_error(table_summary(df, l = 2))
})

test_that("split table contains all data values", {
    out = capture.output(table_summary(df, l = 2))
    for (v in c("A", "B", "C", "D", "E", "10", "50"))
        expect_true(any(grepl(v, out)))
})

test_that("l >= nrow does not split the table", {
    out_normal = capture.output(table_summary(df))
    out_nosplit = capture.output(table_summary(df, l = nrow(df)))
    expect_equal(out_normal, out_nosplit)
})

test_that("split table with header prints without error", {
    expect_no_error(table_summary(df_long, l = 5, header = TRUE))
})

test_that("split table with uneven halves prints without error", {
    expect_no_error(table_summary(df_long, l = 3))
})

# ---- Border char ----

test_that("custom border_char appears in output", {
    out = capture.output(table_summary(df, border_char = "="))
    expect_true(any(grepl("=", out)))
})

test_that("default border_char is not present when overridden", {
    out = capture.output(table_summary(df, border_char = "="))
    expect_false(any(grepl("^-+$", trimws(out))))
})

test_that("unicode border_char works", {
    expect_no_error(table_summary(df, border_char = "\u2500"))
})

# ---- Align ----

test_that("align as single string runs without error", {
    expect_no_error(table_summary(df, align = "center"))
    expect_no_error(table_summary(df, align = "left"))
    expect_no_error(table_summary(df, align = "right"))
})

test_that("align as length-2 vector runs without error", {
    expect_no_error(table_summary(df, align = c("center", "right")))
    expect_no_error(table_summary(df, align = c("left", "center")))
})

test_that("align as named list runs without error", {
    expect_no_error(
        table_summary(df, align = list(left_col = "left", right_col = "right"))
    )
})

test_that("align as partial named list (left_col only) runs without error", {
    expect_no_error(table_summary(df, align = list(left_col = "center")))
})

# ---- sm_style() (named strings) ----

test_that("sm_style() with named strings runs without error", {
    expect_no_error(
        table_summary(
            df,
            header = TRUE,
            style = sm_style(
                left_col = "blue_bold",
                right_col = "red",
                title = "green",
                border_text = "yellow"
            )
        )
    )
})

test_that("sm_style() rejects non-string, non-function values", {
    expect_error(sm_style(left_col = 123), "must be a string or a function")
})

test_that("sm_style() returns correct class", {
    s = sm_style(left_col = "blue")
    expect_s3_class(s, "sm_style")
    expect_s3_class(s, "tabstats_style")
})

test_that("sm_style() with custom sep runs without error", {
    expect_no_error(table_summary(df, style = sm_style(sep = "|")))
})

test_that("custom sep appears in output", {
    out = capture.output(table_summary(df, style = sm_style(sep = "|")))
    expect_true(any(grepl("|", out, fixed = TRUE)))
})

# ---- sm_style() (anonymous functions) ----

test_that("sm_style() with lambda functions runs without error", {
    expect_no_error(
        table_summary(
            df,
            header = TRUE,
            style = sm_style(
                left_col = \(x) cli::col_red(x),
                right_col = \(x) cli::col_blue(x)
            )
        )
    )
})

test_that("sm_style() stores functions correctly", {
    fn = \(x) cli::col_red(x)
    s = sm_style(left_col = fn)
    expect_identical(s$left_col, fn)
})

# ---- print.tabstats_style ----

test_that("print.tabstats_style runs without error", {
    expect_no_error(print(sm_style(left_col = "blue", right_col = "red")))
})

test_that("print.tabstats_style handles empty style object", {
    expect_no_error(print(sm_style()))
})

# ---- Combinations ----

test_that("title + header + split + sm_style + align all together", {
    expect_no_error(
        table_summary(
            df_long,
            title = "Full Test",
            l = 5,
            header = TRUE,
            style = sm_style(left_col = "bold", border_text = "blue"),
            align = c("left", "right")
        )
    )
})

test_that("center_table = TRUE runs without error", {
    expect_no_error(table_summary(df, center_table = TRUE))
})

matrix_spec_resolver_pm = function(m, digits, name) {
    if (!isSymmetric(unname(m)))
        stop("Matrix must be symmetric.", call. = FALSE)

    new_m = as.table(m) |>
        as.data.frame()

    spec = new_pairwise_data(
        var1 = vctrs::vec_cast(new_m[[1]], character()),
        var2 = vctrs::vec_cast(new_m[[2]], character()),
        corr = format(new_m[[3]], digits = digits)
    )
    attr(spec, "title") = name
    spec
}

style_resolver_pm = function(style, extra_names) {
    identity_fn = function(x) x
    all_keys = unique(c(extra_names, "title", "border_text"))
    defaults = stats::setNames(rep(list(identity_fn), length(all_keys)), all_keys)
    resolved = utils::modifyList(defaults, if (is.null(style)) list() else style)

    for (nm in names(resolved))
        if (is.character(resolved[[nm]]))
            resolved[[nm]] = style_fn_pm(resolved[[nm]])

    resolved
}

style_fn_pm = function(style_string) {
    style_map = list(
        red = cli::col_red, blue = cli::col_blue, green = cli::col_green,
        yellow = cli::col_yellow, magenta = cli::col_magenta, cyan = cli::col_cyan,
        white = cli::col_white, black = cli::col_black,
        bold = cli::style_bold, italic = cli::style_italic,
        underline = cli::style_underline
    )
    parts = unlist(strsplit(style_string, "_"))
    fns = lapply(parts, function(p) {
        if (p %in% names(style_map)) style_map[[p]]
        else { warning("Unknown style: '", p, "'. Ignored."); function(x) x }
    })
    Reduce(function(f, g) function(x) f(g(x)), fns, right = TRUE)
}

matrix_styler_pm = function(mat, field_names, style) {
    n_fields = length(field_names)
    styled = mat

    for (i in seq_len(nrow(mat))) {
        field = field_names[(i - 1L) %% n_fields + 1L]
        fn = style[[field]]
        if (is.null(fn)) fn = function(x) x

        for (j in seq_len(ncol(mat))) {
            if (j == 1L || mat[i, j] == "") next
            styled[i, j] = fn(mat[i, j])
        }
    }
    styled
}

left_pad_pm = function(center_table, total_width) {
    if (!center_table) return("")
    term_w = tryCatch(cli::console_width(), error = function(e) as.integer(getOption("width")))
    strrep(" ", max(0L, floor((term_w - total_width) / 2L)))
}

print_layout_pm = function(field_names, style, border_char, layout_center, center_table, name) {
    w = 29L
    top = paste0("\u250C", strrep("\u2500", w - 2L), "\u2510")
    mid = paste0("\u251C", strrep("\u2500", w - 2L), "\u2524")
    bot = paste0("\u2514", strrep(border_char, w - 2L), "\u2518")
    inner = function(txt) paste0("| ", align_center(txt, w - 4L), " |")

    short_name = if (nchar(name) > w - 14L) paste0(substr(name, 1L, w - 17L), "...") else name

    box = c(
        top,
        inner(style[["title"]](paste0("Layout for ", short_name))),
        mid
    )
    for (nm in field_names) {
        label = paste0("< ", nm, " >")
        styled_label = tryCatch(
            style[[nm]](label),
            warning = function(w) NULL,
            error = function(e) NULL
        )
        if (is.null(styled_label)) {
            probe = tryCatch(style[[nm]]("0.001"), error = function(e) NULL)
            styled_label = if (!is.null(probe)) {
                pre = regmatches(probe, regexpr("^(\033\\[[0-9;]*m)+", probe))
                post = regmatches(probe, regexpr("(\033\\[[0-9;]*m)+$", probe))
                if (length(pre) && length(post)) paste0(pre, label, post) else label
            } else {
                label
            }
        }
        box = c(box, inner(styled_label))
    }

    box = c(box, bot)

    term_w = tryCatch(cli::console_width(), error = function(e) as.integer(getOption("width")))
    pad = if (layout_center || center_table)
        strrep(" ", max(0L, floor((term_w - w) / 2L)))
    else
        ""

    cat("\n")
    for (line in box) cat(pad, line, "\n", sep = "")
    cat("\n")
}

spec_lt = new_corr_data(
    var1 = c("a", "a", "b"),
    var2 = c("b", "c", "c"),
    rho = c("0.89", "0.79", "0.66")
)

spec_multi = new_corr_data(
    var1 = c("a", "a", "b"),
    var2 = c("b", "c", "c"),
    rho = c("0.89", "0.79", "0.66"),
    pval = c("<0.001", "<0.001", "0.012")
)

# ---- Input validation ----

test_that("errors on non-matrix, non-corr_spec input", {
    expect_error(corr_matrix(list(a = 1)), "pairwise_spec.*symmetric matrix")
})

test_that("errors on non-symmetric matrix", {
    expect_error(corr_matrix(matrix(1:4, 2, 2)), "symmetric")
})

# ---- corr_spec input -----

test_that("accepts a corr_spec object and returns a matrix invisibly", {
    out = expect_invisible(corr_matrix(spec_lt, title = "Pearson Correlation Matrix"))
    expect_true(is.matrix(out))
    expect_type(out, "character")
})

test_that("accepts a multi-field corr_spec", {
    expect_no_error(corr_matrix(spec_multi, title = "Pearson Correlation Matrix"))
})

# ---- plain matrix input ------------------------------------------------------

test_that("accepts a symmetric matrix", {
    m = cor(mtcars[, 1:4])
    out = expect_invisible(corr_matrix(m, title = "Pearson Correlation Matrix"))
    expect_true(is.matrix(out))
})

test_that("auto-generates variable names when colnames is NULL", {
    m = matrix(c(1, 0.5, 0.5, 1), 2, 2)
    expect_no_error(corr_matrix(m))
})

test_that("uses 'Correlation Matrix' as title when not supplied", {
    out = capture.output(corr_matrix(cor(mtcars[, 1:3])))
    expect_true(any(grepl("Correlation Matrix", out)))
})

test_that("uses supplied title string in output", {
    out = capture.output(corr_matrix(cor(mtcars[, 1:3]), title = "Spearman Correlation Matrix"))
    expect_true(any(grepl("Spearman Correlation Matrix", out)))
})

# ---- diag_1 --------------

test_that("diagonal cells show '1' when diag_1 = TRUE", {
    out = corr_matrix(cor(mtcars[, 1:3]), title = "Pearson Correlation Matrix")
    diag_vals = diag(out[seq(1, nrow(out), 1), ])
    expect_true(any(out == "1"))
})

# ---- layout_view ---------

test_that("layout_view = TRUE prints without error", {
    expect_no_error(corr_matrix(spec_lt, layout_view = TRUE))
})

# ---- cm_style() ----------

test_that("cm_style() with named string runs without error", {
    expect_no_error(
        corr_matrix(spec_multi, style = cm_style(rho = "blue_bold", pval = "red"))
    )
})

test_that("cm_style() with lambda runs without error", {
    expect_no_error(
        corr_matrix(spec_lt, style = cm_style(rho = \(x) cli::col_cyan(x)))
    )
})

test_that("cm_style() returns correct class", {
    s = cm_style(rho = "blue")
    expect_s3_class(s, "cm_style")
    expect_s3_class(s, "tabstats_style")
})

test_that("cm_style() rejects non-string, non-function values", {
    expect_error(cm_style(rho = 123), "must be a string or a function")
})

# ---- style_resolver_pm() -----------------------------------------------------

test_that("fills missing keys with identity functions", {
    style = style_resolver_pm(list(), c("rho", "pval"))
    expect_true(is.function(style[["rho"]]))
    expect_true(is.function(style[["title"]]))
    expect_true(is.function(style[["border_text"]]))
})

test_that("converts string style to a function", {
    style = style_resolver_pm(list(rho = "blue"), c("rho"))
    expect_type(style[["rho"]], "closure")
})

# ---- style_fn_pm() -------

test_that("converts known style string to a function", {
    fn = style_fn_pm("red")
    expect_type(fn, "closure")
})

test_that("warns on unknown style string component", {
    expect_warning(style_fn_pm("notacolor"), "Unknown style")
})

test_that("combined style string (e.g. blue_bold) returns a function", {
    fn = style_fn_pm("blue_bold")
    expect_type(fn, "closure")
})

# ---- new_corr_data() -----

test_that("errors when var1 and var2 have different lengths", {
    expect_error(
        new_corr_data(var1 = c("a", "b"), var2 = "c", rho = c("0.5", "0.6")),
        "same length"
    )
})

test_that("errors when extra vector has wrong length", {
    expect_error(
        new_corr_data(var1 = c("a"), var2 = c("b"), rho = c("0.5", "0.6")),
        "same length"
    )
})

test_that("returns a corr_spec object", {
    expect_s3_class(spec_lt, "corr_spec")
})

test_that("detect_pattern correctly identifies lower triangle", {
    expect_equal(spec_lt$pattern, "lt")
})

test_that("detect_pattern correctly identifies full matrix", {
    vars = c("a", "b")
    all_pairs = expand.grid(var1 = vars, var2 = vars, stringsAsFactors = FALSE)
    spec = new_corr_data(
        var1 = all_pairs$var1,
        var2 = all_pairs$var2,
        rho = rep("0.5", nrow(all_pairs))
    )
    expect_equal(spec$pattern, "full")
})

test_that("extract_vars preserves first-seen order", {
    vars = extract_vars(c("b", "a", "b"), c("a", "c", "c"))
    expect_equal(vars, c("b", "a", "c"))
})

data_formatter = function(
        x,
        digits,
        digits_by_col,
        scientific,
        na_print
) {
    # x = dplyr::mutate(x, dplyr::across(tidyselect::where(is.factor), as.character))
    i = vapply(x, is.factor, logical(1))
    x[i] = lapply(x[i], as.character)

    fmt_num = function(col, col_name) {
        d = digits_by_col[[col_name]] %||% digits
        int_like = all(is.na(col) | col %% 1 == 0)
        out = character(length(col))
        out[is.na(col)] = na_print
        nna = !is.na(col)

        out[nna] = if (int_like) {
            format(col[nna], scientific = FALSE, trim = TRUE)
        } else if (scientific) {
            format(col[nna], digits = d, scientific = TRUE, trim = TRUE)
        } else {
            format(round(col[nna], d), nsmall = d, scientific = FALSE, trim = TRUE)
        }
        out
    }

    for (nm in names(x)[vapply(x, is.numeric, FUN.VALUE = logical(1))])
        x[[nm]] = fmt_num(x[[nm]], nm)

    # x = dplyr::mutate(
    #     x,
    #     dplyr::across(tidyselect::where(\(c) !is.numeric(c) && !is.character(c)), as.character),
    #     dplyr::across(tidyselect::where(\(c) !is.numeric(c)), \(c) ifelse(is.na(c), "NA", c)),
    #     dplyr::across(dplyr::everything(), as.character)
    # )
    # x
    for (i in seq_along(x)) {
        col = x[[i]]

        if (!is.numeric(col) && !is.character(col)) {
            col = as.character(col)
        }

        if (!is.numeric(col)) {
            col[is.na(col)] = "NA"
        }

        x[[i]] = as.character(col)
    }

    x
}

#' Total Widths by Column
#'
#' @keywords internal
#' @noRd
col_widths_total = function(
        x,
        x_chars,
        col_names,
        style_columns,
        original_x,
        min_width
) {
    widths = x |>
        lapply(nchar) |>
        vapply(max, integer(1), na.rm = TRUE) |>
        pmax(nchar(col_names))

    if (is.list(style_columns)) {
        for (i in seq_along(col_names)) {
            fn = style_mapper(style_columns, col_names[i], i)
            if (!is.function(fn)) next

            plain_vals = vapply(seq_len(nrow(x)), function(r) {
                ctx = ctx_maker(x_chars[r, i], col_names[i], i, FALSE, original_x, widths[i])
                res = tryCatch(fn(ctx), error = function(e) x_chars[r, i])
                if (is.character(res) && length(res) == 1)
                    gsub("\033\\[[0-9;]*m", "", res)
                else
                    x_chars[r, i]
            }, character(1))

            widths[i] = max(widths[i], nchar(plain_vals), na.rm = TRUE)
        }
    }

    if (!is.null(min_width) && is.numeric(min_width))
        widths = pmax(widths, floor(min_width[1]))

    widths
}

#' Vertical border resolution
#'
#' @keywords internal
#' @noRd
vb_resolve = function(
        vb,
        col_names,
        col_widths
) {
    char = vb$char %||% getOption("tab_default")$vb_char
    spec = vb$after
    idx = integer(0)

    if (length(spec) > 0) {
        idx = if (is.numeric(spec)) spec else match(spec, col_names)
        idx = sort(unique(idx[!is.na(idx) & idx > 0 & idx < length(col_widths)]))
    }

    list(char = char, idx = idx, n_borders = length(idx))
}

#' Total width calculator
#'
#' @keywords internal
#' @noRd
width_total = function(
        col_widths,
        n_space,
        n_borders,
        vb_char_w
) {
    sum(col_widths) + (length(col_widths) - 1) * n_space + n_borders * (1 + vb_char_w) + 4
}

left_padding = function(center_table, term_width, total_width) {
    if (center_table && term_width > total_width)
        strrep(" ", floor((term_width - total_width) / 2))
    else
        ""
}

#' Optimal horizontal line drawer
#'
#' @keywords internal
#' @noRd
horizontal_line = function(col_widths, n_space, vb_idx, border_char, mid_char = border_char) {
    line = paste0(border_char, border_char)

    for (i in seq_along(col_widths)) {
        line = paste0(line, strrep(border_char, col_widths[i]))
        if (i < length(col_widths)) {
            sep = if (i %in% vb_idx)
                paste0(border_char, mid_char, strrep(border_char, n_space))
            else
                strrep(border_char, n_space)
            line = paste0(line, sep)
        }
    }

    paste0(line, border_char, border_char)
}

line_maker = function(col_widths, n_space, vb_idx, vb_char, border_char, n_borders) {
    if (n_borders == 0) {
        w = sum(col_widths) + max(0, length(col_widths) - 1) * n_space + 4
        line = strrep(border_char, w)
        return(list(top = line, mid = line, bot = line, title = line))
    }

    con = getOption("tab_default")
    list(
        top = horizontal_line(col_widths, n_space, vb_idx, border_char, con$vb_top),
        mid = horizontal_line(col_widths, n_space, vb_idx, border_char, con$vb_mid),
        bot = horizontal_line(col_widths, n_space, vb_idx, border_char, con$vb_bottom),
        title = horizontal_line(col_widths, n_space, integer(0), border_char)
    )
}

title_display = function(title, lines, justify_cols, style_colnames, original_x, pad) {
    inner_w = nchar(lines$title) - 2
    just = if (is.list(justify_cols) && "title" %in% names(justify_cols))
        justify_cols[["title"]]
    else
        "center"
    fmt = justify_text(title, inner_w, just)

    if (is.list(style_colnames) && "title" %in% names(style_colnames)) {
        fn = style_colnames[["title"]]
        if (is.function(fn)) {
            ctx = list(
                value = title,
                formatted_value = fmt,
                col_name = "title",
                col_index = 0,
                is_header = TRUE,
                data = original_x,
                justify = just,
                width = inner_w
            )
            styled = tryCatch(fn(ctx), error = function(e) fmt)
            if (is.character(styled) && length(styled) == 1) fmt = styled
        }
    }

    cat(pad, lines$title, "\n", sep = "")
    cat(pad, " ", fmt, " ", "\n", sep = "")
}

col_header_display = function(col_names, col_widths, justify_cols, n_space,
                              style_colnames, original_x, vb, pad, mid_line) {
    header = format_row(
        stats::setNames(col_names, col_names),
        col_widths, justify_cols, n_space,
        style_colnames, original_x, TRUE, vb
    )
    cat(pad, header, "\n", sep = "")
    cat(pad, mid_line, "\n", sep = "")
}

row_display = function(x_chars, col_names, col_widths, justify_cols, n_space,
                       style_columns, original_x, vb, pad) {
    for (i in seq_len(nrow(x_chars))) {
        row = stats::setNames(x_chars[i, ], col_names)
        cat(
            pad,
            format_row(row, col_widths, justify_cols, n_space, style_columns, original_x, FALSE, vb),
            "\n",
            sep = ""
        )
    }
}

default_table_print = function(
        x_chars, col_names, col_widths, vb_setup,
        justify_cols, n_space, title,
        style_colnames, style_columns, original_x,
        center_table, term_width, total_width, border_char
) {
    lines = line_maker(col_widths, n_space, vb_setup$idx, vb_setup$char, border_char, vb_setup$n_borders)
    pad = left_padding(center_table, term_width, total_width)
    vb = list(char = vb_setup$char, after = vb_setup$idx)

    if (!is.null(title)) {
        title_display(title, lines, justify_cols, style_colnames, original_x, pad)
    }
    cat(pad, lines$top, "\n", sep = "")
    col_header_display(col_names, col_widths, justify_cols, n_space, style_colnames, original_x, vb, pad, lines$mid)
    row_display(x_chars, col_names, col_widths, justify_cols, n_space, style_columns, original_x, vb, pad)
    cat(pad, lines$bot, "\n", sep = "")
}

wrapped_table_print = function(
        x_chars, col_names, col_widths, vb_setup,
        justify_cols, n_space, title,
        style_colnames, style_columns, original_x,
        center_table, term_width, border_char, wrap_threshold
) {
    first_w = col_widths[1] + 4 + n_space
    avail_w = term_width * wrap_threshold - first_w
    col_groups = group_cols(2:length(col_widths), col_widths, avail_w, n_space)

    for (g in seq_along(col_groups)) {
        cur_cols = c(1L, col_groups[[g]])
        cur_names = col_names[cur_cols]
        cur_w = col_widths[cur_cols]
        cur_chars = x_chars[, cur_cols, drop = FALSE]
        cur_vb_idx = vb_remap(vb_setup$idx, cur_cols)
        cur_vb = list(char = vb_setup$char, after = cur_vb_idx)
        cur_nb = length(cur_vb_idx)
        cur_total = width_total(cur_w, n_space, cur_nb, nchar(vb_setup$char))
        pad = left_padding(center_table, term_width, cur_total)
        lines = line_maker(cur_w, n_space, cur_vb_idx, vb_setup$char, border_char, cur_nb)

        if (g == 1) {
            if (!is.null(title))
                title_display(title, lines, justify_cols, style_colnames, original_x, pad)
        } else {
            cat("\n---\n\n")
        }

        cat(pad, lines$top, "\n", sep = "")
        col_header_display(cur_names, cur_w, justify_cols, n_space, style_colnames, original_x, cur_vb, pad, lines$mid)
        row_display(cur_chars, cur_names, cur_w, justify_cols, n_space, style_columns, original_x, cur_vb, pad)
        cat(pad, lines$bot, "\n", sep = "")
    }
}

group_cols = function(cols, col_widths, avail_w, n_space) {
    groups = vector("list", length(cols))
    g = 0L
    cur = integer(length(cols))
    cur_n = 0L
    cur_w = 0

    for (i in cols) {
        w = col_widths[i] + n_space
        if (cur_n == 0L || cur_w + w <= avail_w) {
            cur_n = cur_n + 1L
            cur[cur_n] = i
            cur_w = cur_w + w
        } else {
            g = g + 1L
            groups[[g]] = cur[seq_len(cur_n)]
            cur[1L] = i
            cur_n = 1L
            cur_w = w
        }
    }
    if (cur_n > 0L) {
        g = g + 1L
        groups[[g]] = cur[seq_len(cur_n)]
    }

    groups[seq_len(g)]
}

vb_remap = function(vb_idx, cur_cols) {
    if (length(vb_idx) == 0) return(integer(0))
    mapped = match(vb_idx, cur_cols)
    sort(unique(mapped[!is.na(mapped) & mapped < length(cur_cols)]))
}

ctx_maker = function(value, col_name, col_idx, is_header, data, width) {
    list(
        value = value,
        formatted_value = value,
        col_name = col_name,
        col_index = col_idx,
        is_header = is_header,
        data = data,
        justify = "center",
        width = width
    )
}

style_mapper = function(styles, col_name, col_idx) {
    if (is.null(styles)) return(NULL)
    if (!is.null(names(styles))) {
        if (col_name %in% names(styles)) return(styles[[col_name]])
        if (as.character(col_idx) %in% names(styles)) return(styles[[as.character(col_idx)]])
    }
    if (is.null(names(styles)) && col_idx <= length(styles)) return(styles[[col_idx]])
    NULL
}

small = head(mtcars, 5)

# ---- Input validation --------------------------------------------------------

test_that("errors when x cannot be coerced to data frame", {
    expect_error(table_default(function() NULL), "data frame")
})

test_that("coerces a matrix to data frame silently", {
    m = matrix(1:9, 3, 3, dimnames = list(NULL, c("a", "b", "c")))
    expect_no_error(table_default(m))
})

test_that("errors on invalid style_columns type", {
    expect_error(
        table_default(small, style_columns = list(mpg = "cyan")),
        "td_style"
    )
})

test_that("errors on invalid style_colnames type", {
    expect_error(
        table_default(small, style_colnames = list(mpg = "cyan")),
        "td_style"
    )
})

test_that("warns when n_space is negative and resets to 2", {
    expect_warning(table_default(small, n_space = -1), "n_space")
})

test_that("warns when nrows is negative and resets to 10", {
    expect_warning(table_default(small, nrows = -1), "nrows")
})

# ---- Basic output ------------------------------------------------------------

test_that("returns a data frame invisibly", {
    result = expect_invisible(table_default(small))
    expect_s3_class(result, "data.frame")
})

test_that("output contains column names", {
    out = capture.output(table_default(small))
    expect_true(any(grepl("mpg", out)))
    expect_true(any(grepl("cyl", out)))
})

test_that("prints title when supplied", {
    out = capture.output(table_default(small, title = "My Table"))
    expect_true(any(grepl("My Table", out)))
})

test_that("empty data frame with 0 cols prints message and returns NULL", {
    df = data.frame()
    result = expect_invisible(table_default(df))
    expect_null(result)
})

# ---- Truncation ----

test_that("truncates to nrows rows", {
    result = table_default(mtcars, nrows = 3)
    expect_equal(nrow(result), 3L)
})

test_that("truncation alert is shown when rows are truncated", {
    expect_message(
        table_default(mtcars, nrows = 3),
        regexp = "Showing 3 of 32 rows"
    )
})

# ---- Row names -----

test_that("show_row_names = TRUE adds row_names column", {
    result = table_default(small, show_row_names = TRUE)
    expect_true("row_names" %in% names(result))
})

test_that("show_row_names = FALSE does not add row_names column", {
    result = table_default(small, show_row_names = FALSE)
    expect_false("row_names" %in% names(result))
})

# ---- td_style() ----

test_that("td_style() with named string runs without error", {
    expect_no_error(
        table_default(small, style_columns = td_style(mpg = "cyan", cyl = "magenta"))
    )
})

test_that("td_style() with lambda runs without error", {
    expect_no_error(
        table_default(
            small,
            style_columns = td_style(mpg = \(ctx) cli::col_red(ctx$value))
        )
    )
})

test_that("td_style() rejects non-string, non-function values", {
    expect_error(td_style(mpg = 123), "must be a string or a function")
})

test_that("td_style() returns correct class", {
    s = td_style(mpg = "cyan")
    expect_s3_class(s, "td_style")
    expect_s3_class(s, "tabstats_style")
})

test_that("td_style() stores entries correctly", {
    fn = \(ctx) ctx$value
    s = td_style(mpg = fn)
    expect_identical(s$mpg, fn)
})

# ---- data_formatter() --------------------------------------------------------

test_that("rounds numeric columns to given digits", {
    df = data.frame(x = c(1.23456, 2.34567))
    fmt = data_formatter(df, digits = 2, digits_by_col = NULL,
                         scientific = FALSE, na_print = "")
    expect_equal(fmt$x, c("1.23", "2.35"))
})

test_that("formats integer-like numerics without decimals", {
    df = data.frame(x = c(1.0, 2.0, 3.0))
    fmt = data_formatter(df, digits = 3, digits_by_col = NULL,
                         scientific = FALSE, na_print = "")
    expect_equal(fmt$x, c("1", "2", "3"))
})

test_that("replaces NA with na_print string", {
    df = data.frame(x = c(1.0, NA))
    fmt = data_formatter(df, digits = 1, digits_by_col = NULL,
                         scientific = FALSE, na_print = "-")
    expect_equal(fmt$x[2], "-")
})

test_that("converts factors to character", {
    df = data.frame(f = factor(c("a", "b")))
    fmt = data_formatter(df, digits = 1, digits_by_col = NULL,
                         scientific = FALSE, na_print = "")
    expect_type(fmt$f, "character")
})

test_that("digits_by_col overrides global digits for named column", {
    df = data.frame(x = c(1.23456), y = c(1.23456))
    fmt = data_formatter(df, digits = 3, digits_by_col = list(x = 1),
                         scientific = FALSE, na_print = "")
    expect_equal(fmt$x, "1.2")
    expect_equal(fmt$y, "1.235")
})

test_that("scientific = TRUE formats in scientific notation", {
    df = data.frame(x = c(12345.678))
    fmt = data_formatter(df, digits = 2, digits_by_col = NULL,
                         scientific = TRUE, na_print = "")
    expect_true(grepl("e", fmt$x, ignore.case = TRUE))
})

# ---- col_widths_total() ------------------------------------------------------

test_that("returns widths >= column name lengths", {
    df = data.frame(a = "x", longcolname = "y")
    x_chars = as.matrix(df)
    widths = col_widths_total(df, x_chars, colnames(df), NULL, df, NULL)
    expect_gte(widths[2], nchar("longcolname"))
})

test_that("respects min_width", {
    df = data.frame(a = "x", b = "y")
    x_chars = as.matrix(df)
    widths = col_widths_total(df, x_chars, colnames(df), NULL, df, 20)
    expect_true(all(widths >= 20))
})

# ---- vb_resolve() ------------------------------------------------------------

test_that("returns empty idx when vb has no after spec", {
    setup = vb_resolve(list(), c("a", "b", "c"), c(4L, 4L, 4L))
    expect_length(setup$idx, 0)
})

test_that("maps character column names to indices", {
    setup = vb_resolve(list(after = "b"), c("a", "b", "c"), c(4L, 4L, 4L))
    expect_equal(setup$idx, 2L)
})

test_that("maps numeric indices directly", {
    setup = vb_resolve(list(after = 2L), c("a", "b", "c"), c(4L, 4L, 4L))
    expect_equal(setup$idx, 2L)
})

test_that("excludes out-of-range border indices", {
    setup = vb_resolve(list(after = 10L), c("a", "b", "c"), c(4L, 4L, 4L))
    expect_length(setup$idx, 0)
})

# ---- width_total() -----------------------------------------------------------

test_that("computes correct total width", {
    expect_equal(width_total(c(5L, 5L, 5L), 2L, 0L, 1L), 23L)
})

test_that("accounts for vertical borders in total width", {
    without = width_total(c(5L, 5L), 2L, 0L, 1L)
    with_vb = width_total(c(5L, 5L), 2L, 1L, 1L)
    expect_gt(with_vb, without)
})

# ---- left_padding() ----------------------------------------------------------

test_that("returns empty string when center_table = FALSE", {
    expect_equal(left_padding(FALSE, 120, 80), "")
})

test_that("returns non-empty string when table is narrower than terminal", {
    pad = left_padding(TRUE, 200, 80)
    expect_gt(nchar(pad), 0)
})

test_that("returns empty string when table is wider than terminal", {
    expect_equal(left_padding(TRUE, 80, 200), "")
})

# ---- style_mapper() ----------------------------------------------------------

test_that("returns function from named list by column name", {
    fn = function(ctx) ctx$value
    result = style_mapper(list(mpg = fn), "mpg", 1)
    expect_identical(result, fn)
})

test_that("returns function by column index string", {
    fn = function(ctx) ctx$value
    result = style_mapper(list("1" = fn), "mpg", 1)
    expect_identical(result, fn)
})

test_that("returns NULL for unknown column", {
    expect_null(style_mapper(list(mpg = function(ctx) ctx$value), "cyl", 2))
})

test_that("returns NULL when styles is NULL", {
    expect_null(style_mapper(NULL, "mpg", 1))
})

test_that("returns entry by position for unnamed list", {
    fn = function(ctx) ctx$value
    result = style_mapper(list(fn), "anything", 1)
    expect_identical(result, fn)
})

# ---- ctx_maker() -------------------------------------------------------------

test_that("returns list with all required keys", {
    ctx = ctx_maker("val", "col1", 1L, FALSE, mtcars, 10L)
    expect_named(
        ctx,
        c("value", "formatted_value", "col_name", "col_index",
          "is_header", "data", "justify", "width"),
        ignore.order = TRUE
    )
})

test_that("value and formatted_value are equal on construction", {
    ctx = ctx_maker("val", "col1", 1L, FALSE, mtcars, 10L)
    expect_equal(ctx$value, ctx$formatted_value)
})

# ---- group_cols() ------------------------------------------------------------

test_that("puts all columns in one group when they fit", {
    groups = group_cols(2:5, c(5L, 5L, 5L, 5L, 5L), avail_w = 100, n_space = 2)
    expect_length(groups, 1)
    expect_equal(groups[[1]], 2:5)
})

test_that("splits into multiple groups when columns overflow", {
    groups = group_cols(2:5, rep(10L, 5), avail_w = 14, n_space = 2)
    expect_gt(length(groups), 1)
})

test_that("each group always starts a new column when avail_w is tiny", {
    groups = group_cols(2:4, rep(10L, 4), avail_w = 1, n_space = 2)
    expect_length(groups, 3)
})

# ---- vb_remap() ----

test_that("returns empty when vb_idx is empty", {
    expect_equal(vb_remap(integer(0), 1:4), integer(0))
})

test_that("remaps border index to position within cur_cols", {
    expect_equal(vb_remap(3L, c(1L, 3L, 5L)), 2L)
})

test_that("excludes last column position", {
    expect_length(vb_remap(5L, c(1L, 3L, 5L)), 0)
})

test_that("handles multiple remapped indices", {
    result = vb_remap(c(2L, 4L), c(1L, 2L, 4L, 6L))
    expect_equal(result, c(2L, 3L))
})

# ---- Justify Columns ----
test_that("Justify columns", {
    # Bare column
    expect_no_error({
        table_default(
            head(iris, 5),
            justify_cols = list(Sepal.Length = "left")
        )
    })
    # Indexed column
    expect_no_error({
        table_default(
            head(iris, 5),
            justify_cols = list("1" = "left")
        )
    })
    # Right Alignment
    expect_no_error({
        table_default(
            head(iris, 5),
            justify_cols = list(Sepal.Length = "right")
        )
    })
    # Center Alignment
    expect_no_error({
        table_default(
            head(iris, 5),
            justify_cols = list(Sepal.Length = "center")
        )
    })
})

#' Build a pairwise matrix display specification
#'
#' Constructs a structured spec object consumed by `pairwise_matrix()`. Always
#' requires `var1` and `var2` — the pair pattern they encode determines which
#' triangle(s) of the matrix are filled:
#'
#' | Pattern          | Fills                         |
#' |------------------|-------------------------------|
#' | `var1 < var2`    | Lower triangle only           |
#' | `var1 <= var2`   | Lower triangle + diagonal     |
#' | `var1 > var2`    | Upper triangle only           |
#' | `var1 >= var2`   | Upper triangle + diagonal     |
#' | `var1 != var2`   | Both triangles, no diagonal   |
#' | `var1 == var2`   | Full matrix (diag forced to 1)|
#'
#' All additional named vectors become display rows inside each cell, rendered
#' in the order they are supplied.
#'
#' @param var1 Character vector of first variable names per pair.
#' @param var2 Character vector of second variable names per pair.
#' @param ... Named character vectors of equal length to `var1`/`var2`.
#'   Each becomes one display row inside the cell (e.g. `rho`, `pval`, `bf`).
#'
#' @return An object of class `pairwise_spec`.
#'
#' @examples
#' new_pairwise_data(
#'     var1 = c("a", "a", "b"),
#'     var2 = c("b", "c", "c"),
#'     rho = c("0.89", "0.79", "0.66"),
#'     pval = c("<0.001", "<0.001", "<0.001")
#' )
#'
#' @export
new_pairwise_data = function(var1, var2, ...) {
    stopifnot(is.character(var1), is.character(var2))

    n_pairs = length(var1)
    if (length(var2) != n_pairs)
        stop("`var1` and `var2` must be the same length.", call. = FALSE)

    extras = lapply(list(...), as.character)
    for (nm in names(extras)) {
        if (length(extras[[nm]]) != n_pairs)
            stop(
                sprintf("`%s` must have the same length as `var1` (%d).", nm, n_pairs),
                call. = FALSE
            )
    }

    pattern = detect_pattern(var1, var2, n_pairs)
    vars = extract_vars(var1, var2)

    structure(
        list(
            var1 = var1,
            var2 = var2,
            extras = extras,
            pattern = pattern,
            vars = vars
        ),
        class = "pairwise_spec"
    )
}

#' @rdname new_pairwise_data
#' @export
new_corr_data = function(var1, var2, ...) {
    spec = new_pairwise_data(var1, var2, ...)
    class(spec) = c("corr_spec", "pairwise_spec")
    spec
}

detect_pattern = function(var1, var2, n_pairs) {
    all_vars = unique(c(var1, var2))
    n = length(all_vars)

    expected = list(
        lt = n * (n - 1L) / 2L,
        lte = n * (n + 1L) / 2L,
        gt = n * (n - 1L) / 2L,
        gte = n * (n + 1L) / 2L,
        neq = n * (n - 1L),
        all = n * n
    )

    sorted_vars = sort(all_vars)
    idx1 = match(var1, sorted_vars)
    idx2 = match(var2, sorted_vars)
    has_lt = any(idx1 < idx2)
    has_gt = any(idx1 > idx2)
    has_eq = any(var1 == var2)

    if (!has_lt && !has_gt &&  has_eq && n_pairs == expected$all) "full"
    else if ( has_lt && has_gt && has_eq && n_pairs == expected$all) "full"
    else if ( has_lt && has_gt && !has_eq && n_pairs == expected$neq) "neq"
    else if ( has_lt && !has_gt && !has_eq && n_pairs == expected$lt) "lt"
    else if ( has_lt && !has_gt && has_eq && n_pairs == expected$lte) "lte"
    else if (!has_lt && has_gt && !has_eq && n_pairs == expected$gt) "gt"
    else if (!has_lt && has_gt && has_eq && n_pairs == expected$gte) "gte"
    else stop(
        sprintf(
            "Vector length (%d) does not match any valid pattern for %d variable(s).\n",
            n_pairs, n
        ),
        "Expected lengths: lt/gt = ", expected$lt,
        ", lte/gte = ", expected$lte,
        ", neq = ", expected$neq,
        ", full = ", expected$all,
        call. = FALSE
    )
}

extract_vars = function(var1, var2) {
    seen = character(0)
    for (v in c(var1, var2))
        if (!v %in% seen) seen = c(seen, v)
    seen
}

digits_resolver = function(digits) {
    defaults = getOption("tab_digits")
    if (is.null(digits)) return(defaults)
    if (is.numeric(digits) && length(digits) == 1)
        return(list(ex = digits, row_pct = digits, col_pct = digits, total_pct = digits))
    if (is.list(digits))
        return(utils::modifyList(defaults, digits))
    defaults
}

pct_resolver = function(percentage) {
    if (is.null(percentage) || length(percentage) == 0 || identical(percentage, FALSE))
        return(list(row = FALSE, col = FALSE, total = FALSE))
    if (identical(percentage, TRUE) || any(c("all", "ALL") %in% percentage))
        return(list(row = TRUE, col = TRUE, total = TRUE))
    list(
        row = any(c("by_row", "row", "rows") %in% percentage),
        col = any(c("by_col", "col", "cols", "column", "columns") %in% percentage),
        total = any(c("by_total", "total") %in% percentage)
    )
}

style_resolver = function(style) {
    identity_fn = function(ctx) ctx$formatted_text
    defaults = list(
        observed = identity_fn,
        expected = identity_fn,
        row_percentage = identity_fn,
        col_percentage = identity_fn,
        total_percentage = identity_fn,
        total = identity_fn,
        title = identity_fn,
        border = options("tab_default")$tab_default$border_char,
        border_text = identity_fn
    )
    utils::modifyList(defaults, if (is.null(style)) list() else style)
}

styler_ct = function(
        style_item,
        text,
        type,
        value = NULL,
        is_total = FALSE
) {
    ctx = list(
        formatted_text = text,
        value = if (is.null(value)) text else value,
        type = type,
        is_total = is_total,
        raw = TRUE
    )
    if (is.function(style_item)) {
        res = tryCatch(style_item(ctx), error = function(e) text)
        return(if (is.character(res)) res else text)
    }
    if (is.character(style_item) && requireNamespace("cli", quietly = TRUE)) {
        for (part in unlist(strsplit(style_item, "_"))) {
            fn = tryCatch(
                if (existsFunction(paste0("col_", part), "cli")) get(paste0("col_", part), envir = asNamespace("cli"))
                else if (existsFunction(paste0("style_", part), "cli")) get(paste0("style_", part), envir = asNamespace("cli"))
                else if (existsFunction(part, "cli")) get(part, envir = asNamespace("cli"))
                else NULL,
                error = function(e) NULL
            )
            if (!is.null(fn)) text = tryCatch(fn(text), error = function(e) text)
        }
    }
    text
}

existsFunction = function(name, pkg) {
    exists(name, where = asNamespace(pkg), mode = "function")
}

pct_formatter_ct = function(x, d) {
    if (is.na(x) || x == 100) return(NULL)
    sprintf(paste0("%.", d, "f%%"), x)
}

num_formatter_ct = function(x, d = 1) {
    if (abs(x - round(x)) < 1e-10) sprintf("%d", as.integer(round(x)))
    else sprintf(paste0("%.", d, "f"), x)
}

cell_width_ct = function(cell) {
    if (!is.character(cell) || cell == "") return(0L)
    max(nchar(strip_ansi(unlist(strsplit(cell, "\n")))))
}

format_cell_ct = function(
        obs,
        exp,
        row_pct,
        col_pct,
        grand_pct,
        is_total,
        show_expected,
        pct_flags,
        digits,
        style
) {
    obs_text = sprintf("%d", as.integer(obs))
    if (show_expected && !is_total) {
        exp_text = sprintf("(%s)", num_formatter_ct(exp, digits$ex))
        main = paste(
            styler_ct(style$observed, obs_text, "observed", obs, is_total),
            styler_ct(style$expected, exp_text, "expected", exp, is_total)
        )
    } else {
        s = if (is_total) style$total else style$observed
        main = styler_ct(s, obs_text, if (is_total) "total" else "observed", obs, is_total)
    }

    lines = main
    if (pct_flags$row && !is.null(txt <- pct_formatter_ct(row_pct, digits$row_pct)))
        lines = paste0(lines, "\n", styler_ct(style$row_percentage, txt, "row_percentage", row_pct, is_total))
    if (pct_flags$col && !is.null(txt <- pct_formatter_ct(col_pct, digits$col_pct)))
        lines = paste0(lines, "\n", styler_ct(style$col_percentage, txt, "col_percentage", col_pct, is_total))
    if (pct_flags$total && !is.null(txt <- pct_formatter_ct(grand_pct, digits$total_pct)))
        lines = paste0(lines, "\n", styler_ct(style$total_percentage, txt, "total_percentage", grand_pct, is_total))

    lines
}

print_ct_layout = function(
        show_expected,
        pct_flags,
        style,
        layout_center,
        center_table
) {
    w = 36L
    top = paste0("\u250C", strrep("\u2500", w - 2L), "\u2510")
    mid = paste0("\u251C", strrep("\u2500", w - 2L), "\u2524")
    bot = paste0("\u2514", strrep(getOption("tab_default")$border_char, w - 2L), "\u2518")
    inner = function(txt) paste0("| ", center_text_x2(txt, w - 4L), " |")

    obs_styled = styler_ct(style$observed, "< Freq >", "observed")
    header = if (show_expected) {
        paste(obs_styled, styler_ct(style$expected, "(< Expected Value >)", "expected"))
    } else {
        obs_styled
    }

    box = c(
        top,
        inner(styler_ct(style$title, "Layout for Cont. Table", "title")),
        mid,
        inner(header)
    )
    if (pct_flags$row)
        box = c(box, inner(styler_ct(style$row_percentage, "< % by total row >", "row_percentage")))
    if (pct_flags$col)
        box = c(box, inner(styler_ct(style$col_percentage, "< % by total column >", "col_percentage")))
    if (pct_flags$total)
        box = c(box, inner(styler_ct(style$total_percentage, "< % by grand total >", "total_percentage")))

    box = c(box, bot)

    cat("\n")
    for (line in box) {
        if (layout_center || center_table) {
            term_w = tryCatch(
                cli::console_width(),
                error = function(e) as.integer(getOption("width"))
            )
            pad = strrep(" ", max(0L, floor((term_w - nchar(strip_ansi(line))) / 2L)))
            cat(pad, line, "\n", sep = "")
        } else {
            cat(line, "\n")
        }
    }
    cat("\n")
}

col_formatter_ct = function(text, width, j) {
    if (j == 1) sprintf("  %-*s", width, text)
    else center_text_x2(text, width)
}

line_builder_ct = function(
        combined,
        all_col_names,
        col_widths,
        main_col,
        main_col_width,
        total_width,
        header_underline,
        center_table,
        hline,
        style,
        border_char,
        pct_flags
) {
    nc = ncol(combined)
    nr = nrow(combined)

    title_text = paste("Cross Tabulation:", all_col_names[1], "by", main_col)
    title_styled = center_text_x2(styler_ct(style$title, title_text, "title"), total_width)

    header_row = paste(
        vapply(seq_len(nc), function(j) {
            col_formatter_ct(all_col_names[j], col_widths[j], j)
        }, character(1)),
        collapse = "   "
    )

    col_label = sprintf("%-*s%s", col_widths[1], "", center_text_x2(main_col, main_col_width))

    sub_hline = if (header_underline) {
        prefix = sprintf("  %-*s", col_widths[1] + 1L, "")
        paste0(prefix, strrep(options("tab_default")$tab_default[[5]], main_col_width - 4L), "  ")
    } else {
        hline
    }

    lines = c(title_styled, hline, col_label, sub_hline, paste0(header_row, "  "), hline)

    for (i in seq_len(nr - 1L)) {
        lines = c(lines, row_renderer_ct(combined, i, col_widths, style, is_total = FALSE))
        if (i < nr - 1L) lines = c(lines, "")
    }

    lines = c(lines, hline, row_renderer_ct(combined, nr, col_widths, style, is_total = TRUE), hline)
    lines
}

row_renderer_ct = function(
        combined,
        i,
        col_widths,
        style,
        is_total
) {
    nc = ncol(combined)
    cell_lines = lapply(seq_len(nc), function(j) {
        unlist(strsplit(as.character(combined[i, j]), "\n"))
    })
    n_lines = max(vapply(cell_lines, length, integer(1)))

    vapply(seq_len(n_lines), function(ln) {
        parts = vapply(seq_len(nc), function(j) {
            txt = if (ln <= length(cell_lines[[j]])) cell_lines[[j]][ln] else ""
            if (j == 1 && ln == 1) {
                txt = styler_ct(
                    style$x_side, txt, "x_side",
                    value = combined[i, 1], is_total = is_total
                )
            }
            col_formatter_ct(txt, col_widths[j], j)
        }, character(1))
        paste0(paste(parts, collapse = "   "), "  ")
    }, character(1))
}

m = matrix(
    c(10, 20, 30, 40),
    nrow = 2,
    dimnames = list(c("Yes", "No"), c("A", "B"))
)

m_named = matrix(
    c(10, 20, 30, 40),
    nrow = 2,
    dimnames = list(
        c("Yes", "No"),
        c("A", "B")
    )
)

# ---- pct_resolver() ----

test_that("returns all FALSE when percentage = NULL", {
    flags = pct_resolver(NULL)
    expect_false(flags$row)
    expect_false(flags$col)
    expect_false(flags$total)
})

test_that("returns all FALSE when percentage = FALSE", {
    flags = pct_resolver(FALSE)
    expect_false(flags$row)
    expect_false(flags$col)
    expect_false(flags$total)
})

test_that("returns all TRUE when percentage = TRUE", {
    flags = pct_resolver(TRUE)
    expect_true(flags$row)
    expect_true(flags$col)
    expect_true(flags$total)
})

test_that("returns all TRUE for 'all'", {
    flags = pct_resolver("all")
    expect_true(flags$row)
    expect_true(flags$col)
    expect_true(flags$total)
})

test_that("handles 'by_row' correctly", {
    flags = pct_resolver("by_row")
    expect_true(flags$row)
    expect_false(flags$col)
    expect_false(flags$total)
})

test_that("handles 'by_col' correctly", {
    flags = pct_resolver("by_col")
    expect_false(flags$row)
    expect_true(flags$col)
    expect_false(flags$total)
})

test_that("handles 'by_total' correctly", {
    flags = pct_resolver("by_total")
    expect_false(flags$row)
    expect_false(flags$col)
    expect_true(flags$total)
})

test_that("handles vector of multiple options", {
    flags = pct_resolver(c("by_row", "by_col"))
    expect_true(flags$row)
    expect_true(flags$col)
    expect_false(flags$total)
})

# ---- digits_resolver() -------------------------------------------------------

test_that("expands scalar to all fields", {
    resolved = digits_resolver(2)
    expect_equal(resolved$ex, 2)
    expect_equal(resolved$row_pct, 2)
    expect_equal(resolved$col_pct, 2)
    expect_equal(resolved$total_pct, 2)
})

test_that("returns defaults when digits = NULL", {
    resolved = digits_resolver(NULL)
    expect_type(resolved, "list")
    expect_true("ex" %in% names(resolved))
})

test_that("partial list overrides only specified fields", {
    defaults = getOption("tab_digits")
    resolved = digits_resolver(list(ex = 3))
    expect_equal(resolved$ex, 3)
    expect_equal(resolved$row_pct, defaults$row_pct)
})

# ---- num_formatter_ct() ------------------------------------------------------

test_that("formats whole numbers without decimals", {
    expect_equal(num_formatter_ct(5.0), "5")
    expect_equal(num_formatter_ct(0), "0")
})

test_that("formats non-integer to given decimal places", {
    expect_equal(num_formatter_ct(3.14159, 2), "3.14")
})

# ---- pct_formatter_ct() ------------------------------------------------------

test_that("returns NULL for 100 (total row/col suppressed)", {
    expect_null(pct_formatter_ct(100, 1))
})

test_that("returns NULL for NA", {
    expect_null(pct_formatter_ct(NA, 1))
})

test_that("formats a percentage correctly", {
    expect_equal(pct_formatter_ct(50, 1), "50.0%")
})

# ---- cell_width_ct() ---------------------------------------------------------

test_that("returns 0 for empty string", {
    expect_equal(cell_width_ct(""), 0L)
})

test_that("returns max line width for multi-line cell", {
    expect_equal(cell_width_ct("10\n50.0%\n25.0%"), 5L)
})

test_that("handles single-line cell", {
    expect_equal(cell_width_ct("123"), 3L)
})

# ---- cross_table() -----

test_that("returns invisible matrix", {
    result = expect_invisible(cross_table(m, layout = FALSE))
    expect_true(is.matrix(result))
})

test_that("accepts a table object", {
    tbl = table(mtcars$cyl, mtcars$gear)
    result = expect_invisible(cross_table(tbl, layout = FALSE))
    expect_true(is.matrix(result))
})

test_that("output contains row names", {
    out = capture.output(cross_table(m_named, layout = FALSE))
    expect_true(any(grepl("Yes|No", out)))
})

test_that("output contains column names", {
    out = capture.output(cross_table(m_named, layout = FALSE))
    expect_true(any(grepl("A|B", out)))
})

test_that("output contains TOTAL label", {
    out = capture.output(cross_table(m, layout = FALSE))
    expect_true(any(grepl("TOTAL", out)))
})

test_that("output includes percentage signs when percentage = 'all'", {
    out = capture.output(cross_table(m, percentage = "all", layout = FALSE))
    expect_true(any(grepl("%", out)))
})

test_that("output has no percentage signs when percentage = FALSE", {
    out = capture.output(cross_table(m, percentage = FALSE, layout = FALSE))
    expect_false(any(grepl("%", out)))
})

test_that("expected = FALSE removes parentheses from output", {
    out = capture.output(cross_table(m, expected = FALSE, layout = FALSE))
    expect_false(any(grepl("\\(", out)))
})

test_that("expected = TRUE shows parenthesised values", {
    out = capture.output(cross_table(m, expected = TRUE, layout = FALSE))
    expect_true(any(grepl("\\(", out)))
})

test_that("layout = TRUE prints without error", {
    expect_no_error(cross_table(m, layout = TRUE))
})

# ---- ct_style() --------

test_that("ct_style() with named strings runs without error", {
    expect_no_error(
        cross_table(
            m,
            layout = FALSE,
            style = ct_style(observed = "blue", expected = "yellow")
        )
    )
})

test_that("ct_style() with lambda runs without error", {
    expect_no_error(
        cross_table(
            m,
            layout = FALSE,
            style = ct_style(observed = \(ctx) cli::col_green(ctx$formatted_text))
        )
    )
})

test_that("ct_style() rejects non-string, non-function values", {
    expect_error(ct_style(observed = 123), "must be a string or a function")
})

test_that("ct_style() returns correct class", {
    s = ct_style(observed = "blue")
    expect_s3_class(s, "ct_style")
    expect_s3_class(s, "tabstats_style")
})

test_that("ct_style() stores entries correctly", {
    fn = \(ctx) ctx$formatted_text
    s = ct_style(observed = fn)
    expect_identical(s$observed, fn)
})

test_that("ct_style() with all keys runs without error", {
    expect_no_error(
        ct_style(
            observed = "blue",
            expected = "yellow",
            row_percentage = "green",
            col_percentage = "cyan",
            total_percentage = "magenta",
            total = "bold",
            title = "bold",
            border_text = "yellow"
        )
    )
})

#' Summarize and Display a Two-Column Data Frame as a Formatted Table
#'
#' This function takes a two-column data frame and formats it into a summary-like table.
#' The table can be optionally split into two parts, centered, and given a title.
#' It is useful for displaying summary information in a clean, tabular format.
#' The function also supports styling with ANSI colors and text formatting through
#' the `{cli}` package and column alignment options.
#'
#' @param data A data frame with exactly two columns. The data to be summarized and displayed.
#' @param title A character string. An optional title to be displayed above the table.
#' @param l An integer. The number of rows to include in the left part of a split table.
#'           If `NULL`, the table is not split.
#' @param header A logical value. If `TRUE`, the column names of `data` are displayed as a header.
#' @param center_table A logical value. If `TRUE`, the table is centered in the terminal.
#' @param border_char Character used for borders. Default is `"\u2500"`.
#' @param style A list controlling the visual styling of table elements using ANSI formatting.
#'   Can include the following components:
#'   - `left_col`: Styling for the left column values.
#'   - `right_col`: Styling for the right column values.
#'   - `border_text`: Styling for the border.
#'   - `title`: Styling for the title.
#'   - `sep`: Separator character between left and right column.
#'
#'   Each style component can be either a predefined style string (e.g., "blue", "red_italic", "bold")
#'   or a function that takes a context list with/without a `value` element and returns the styled text.
#' @param align Controls the alignment of column values.
#'   Can be specified in three ways:
#'   - A single string: affects only the left column (e.g., "left", "center", "right").
#'   - A vector of two strings: affects both columns in order (e.g., c("left", "right")).
#'   - A list with named components: explicitly specifies alignment for each column
#'
#' @param ... Additional arguments (currently unused).
#'
#' @return This function does not return a value. It prints the formatted table to the console.
#'
#' @examples
#' # Create a sample data frame
#' df = data.frame(
#'     Category = c("A", "B", "C", "D", "E"),
#'     Value = c(10, 20, 30, 40, 50)
#' )
#'
#' # Display the table with a title and header
#' table_summary(df, title = "Sample Table", header = TRUE)
#'
#' # Split the table after the second row and center it
#' table_summary(df, l = 2, center_table = TRUE)
#'
#' # Use styling and alignment
#' table_summary(
#'     df,
#'     header = TRUE,
#'     style = list(
#'         left_col = "blue_bold",
#'         right_col = "red",
#'         title = "green",
#'         border_text = "yellow"
#'     ),
#'     align = c("center", "right")
#' )
#'
#' # Use custom styling with lambda functions
#' table_summary(
#'     df,
#'     header = TRUE,
#'     style = sm_style(
#'         left_col = \(ctx) cli::col_red(ctx), # ctx$value is another option
#'         right_col = \(ctx) cli::col_blue(ctx)
#'     ),
#'     align = list(left_col = "left", right_col = "right")
#' )
#'
#' @export
table_summary =
    function(
        data,
        title = NULL,
        l = NULL,
        header = FALSE,
        center_table = FALSE,
        border_char = "-",
        style = list(), align = NULL, ...
    ) {

        if (!is.data.frame(data) || ncol(data) != 2)
            cli::cli_abort("Input must be a data frame with exactly 2 columns.")

        if (!is.null(style) && !is.list(style))
            cli::cli_abort("{.arg style} must be a list or a style object (e.g. {.fn sm_style}).")

        if (inherits(style, "tabstats_style") && !inherits(style, "sm_style")) {
            cli::cli_abort(
                "{.arg style} must be an {.cls sm_style} object for {.fn table_summary}.",
                "x" = "Got {.cls {class(style)[1]}}."
            )
        }

        col_names = colnames(data)
        data_matrix = as.matrix(data)
        sep_width = if (!is.null(style$sep)) nchar(paste0(" ", style$sep, " ")) else 4

        w = compute_widths(data_matrix, col_names, l, sep_width)

        # ---- Border line ----
        h_line = strrep(border_char, w$full_width)
        styled_h_line =
            if (!is.null(style$border_text)) {
                apply_style(style$border_text, h_line)
            } else {
                h_line
            }

        # ---- Optional centering prefix ----
        prefix = ""
        if (center_table) {
            term_width = tryCatch(
                as.numeric(system("tput cols", intern = TRUE)),
                error = function(e)
                    as.double(options("width"))
            )
            prefix = strrep(" ", max(0L, floor((term_width - w$full_width) / 2)))
        }

        # ---- Title ----
        if (!is.null(title)) {
            fmt_title = center_text(title, w$full_width)
            if (!is.null(style$title))
                fmt_title = apply_style(style$title, fmt_title)

            cat("\n", prefix, fmt_title, "\n", sep = "")
        }

        cat(prefix, styled_h_line, "\n", sep = "")

        # ---- Header row ----
        if (header) {
            header_row = if (w$is_split) {
                paste0(
                    format_row_summary(
                        col_names[1], col_names[2],
                        w$left_left_width, w$left_right_width,
                        style = NULL
                    ),
                    "  ",
                    format_row_summary(
                        col_names[1], col_names[2],
                        w$right_left_width, w$right_right_width,
                        style = NULL
                    )
                )
            } else {
                format_row_summary(
                    col_names[1], col_names[2],
                    w$left_width, w$right_width,
                    style = NULL
                )
            }
            cat(prefix, header_row, "\n", sep = "")
            cat(prefix, styled_h_line, "\n", sep = "")
        }

        # ---- Data rows ----
        if (!w$is_split) {
            for (i in seq_len(nrow(data_matrix))) {
                cat(
                    prefix,
                    format_row_summary(
                        data_matrix[i, 1], data_matrix[i, 2],
                        w$left_width, w$right_width,
                        align = align,
                        style = style
                    ),
                    "\n",
                    sep = ""
                )
            }
        } else {
            n_rows_out = max(nrow(w$left_table), nrow(w$right_table))
            for (i in seq_len(n_rows_out)) {
                lr = if (i <= nrow(w$left_table))  w$left_table[i, ]  else c("", "")
                rr = if (i <= nrow(w$right_table)) w$right_table[i, ] else c("", "")

                cat(
                    prefix,
                    paste0(
                        format_row_summary(
                            lr[1], lr[2],
                            w$left_left_width, w$left_right_width,
                            align = align,
                            style = style
                        ),
                        "  ",
                        format_row_summary(
                            rr[1], rr[2],
                            w$right_left_width, w$right_right_width,
                            align = align,
                            style = style
                        )
                    ),
                    "\n",
                    sep = ""
                )
            }
        }

        cat(prefix, styled_h_line, "\n", sep = "")
        invisible(NULL)
    }

align_center = function(text, width, pos = FALSE) {
    clean_text = gsub("\033\\[[0-9;]*m", "", text)
    padding = if (pos) max(width - nchar(clean_text), 0) else width - nchar(clean_text)

    if (padding < 0) return(substr(text, 1, width))

    paste0(strrep(" ", floor(padding / 2)), text, strrep(" ", ceiling(padding / 2)))
}

format_row_pm = function(
        row,
        col_widths,
        left_align_first = FALSE,
        pos = FALSE
) {
    row = as.character(row)
    formatted = vapply(seq_along(row), function(i) {
        if (i == 1 && left_align_first)
            format(row[i], width = col_widths[i], justify = "left")
        else
            align_center(row[i], col_widths[i], pos = pos)
    }, character(1))
    paste0("  ", paste(formatted, collapse = "   "), "  ")
}

align_text = function(text, width, alignment = "left") {
    switch(
        alignment,
        left = sprintf("%-*s", width, text),
        right = sprintf("%*s",  width, text),
        center = {
            padding   = width - nchar(text)
            left_pad  = floor(padding / 2)
            right_pad = ceiling(padding / 2)
            paste0(strrep(" ", left_pad), text, strrep(" ", right_pad))
        },
        sprintf("%-*s", width, text)
    )
}

apply_style = function(style, text) {
    out = if (is.function(style)) {
        style(text)
    } else {
        switch(
            style,
            bold = cli::style_bold(text),
            italic = cli::style_italic(text),
            blue = cli::col_blue(text),
            red = cli::col_red(text),
            green = cli::col_green(text),
            yellow = cli::col_yellow(text),
            blue_bold = cli::col_blue(cli::style_bold(text)),
            red_italic = cli::col_red(cli::style_italic(text)),
            text
        )
    }

    out
}

resolve_alignment = function(align) {
    left_align = "left"
    right_align = "right"

    if (!is.null(align)) {
        if (is.character(align) && length(align) == 1) {
            left_align = right_align = align
        } else if (is.character(align) && length(align) == 2) {
            left_align = align[1]
            right_align = align[2]
        } else if (is.list(align)) {
            if (!is.null(align$left_col))  left_align = align$left_col
            if (!is.null(align$right_col)) right_align = align$right_col
        }
    }

    list(left = left_align, right = right_align)
}

compute_widths = function(data_matrix, col_names, l = NULL, sep_width = 4) {
    n_rows = nrow(data_matrix)
    is_split = !is.null(l) && l < n_rows

    if (!is_split) {
        lw = max(nchar(data_matrix[, 1]), nchar(col_names[1]))
        rw = max(nchar(data_matrix[, 2]), nchar(col_names[2]))
        return(list(
            is_split   = FALSE,
            full_width = lw + rw + 4 + sep_width,
            left_width = lw, right_width = rw
        ))
    }

    lt = data_matrix[1:l, ]
    rt = data_matrix[(l + 1):n_rows, ]

    ll = max(nchar(lt[, 1]), nchar(col_names[1]))
    lr = max(nchar(lt[, 2]), nchar(col_names[2]))
    rl = max(nchar(rt[, 1]), nchar(col_names[1]))
    rr = max(nchar(rt[, 2]), nchar(col_names[2]))

    list(
        is_split = TRUE,
        full_width = (ll + lr + 4 + sep_width) + (rl + rr + 4 + sep_width),
        left_table = lt,
        right_table = rt,
        left_left_width = ll,
        left_right_width = lr,
        right_left_width = rl,
        right_right_width = rr
    )
}

format_row_summary = function(left, right, left_width, right_width, align = NULL, style = NULL) {
    al = resolve_alignment(align)

    left_fmt = align_text(left, left_width, al$left)
    right_fmt = align_text(right, right_width, al$right)

    if (!is.null(style$left_col)) left_fmt = apply_style(style$left_col, left_fmt)
    if (!is.null(style$right_col)) right_fmt = apply_style(style$right_col, right_fmt)

    sep = if (!is.null(style$sep)) paste0(" ", style$sep, " ") else "    "
    paste0("  ", left_fmt, sep, right_fmt)
}

center_text = function(text, width, pos = FALSE) {
    if (pos) {
        padding = pmax(width - nchar(text), 0)
    } else {
        padding = width - nchar(text)
    }
    left_pad = floor(padding / 2)
    right_pad = ceiling(padding / 2)
    paste0(paste0(rep(" ", left_pad), collapse = ""), text, paste0(rep(" ", right_pad), collapse = ""))
}

justify_text = function(text, width, justify = "center") {
    justify = rep_len(justify, length(text))

    width = pmax(width, 0)

    side = vctrs::vec_if_else(
        justify == "left", "right",
        vctrs::vec_if_else(justify == "right", "left", "both")
    )

    vctrs::vec_c(
        !!!mapply(
            function(txt, w, s) {
                pad = w - nchar(txt)
                switch(
                    s,
                    right = paste0(txt, strrep(" ", pad)),
                    left = paste0(strrep(" ", pad), txt),
                    both = paste0(strrep(" ", floor(pad / 2)), txt, strrep(" ", ceiling(pad / 2)))
                )
            },
            text,
            width,
            side,
            SIMPLIFY = FALSE,
            USE.NAMES = FALSE
        )
    )
}

format_row =
    function(
        row,
        col_widths,
        justify_cols = NULL,
        n_space = 2,
        styles = NULL,
        col_data = NULL, is_header = FALSE, vb = list()
    ) {

    original_names = names(row)
    if (is.null(original_names) && length(row) > 0) {
        original_names = as.character(seq_along(row))
    } else if (length(original_names) != length(row)) {
        original_names = as.character(seq_along(row))
    }

    styled_row = character(length(row))
    raw_content = character(length(row))

    for (i in seq_along(row)) {
        col_name_or_index = original_names[i]
        value = row[i]
        raw_content[i] = value

        if (!is.null(styles)) {
            style_fn_or_name = NULL
            lookup_key = if (is_header) value else col_name_or_index

            if (is.list(styles)) {
                if (!is.null(names(styles))) {
                    if (!is.null(lookup_key) && lookup_key %in% names(styles)) {
                        style_fn_or_name = styles[[lookup_key]]
                    } else if (as.character(i) %in% names(styles)) {
                        style_fn_or_name = styles[[as.character(i)]]
                    }
                }
                if (is.null(style_fn_or_name) && is.null(names(styles)) && i <= length(styles)) {
                    style_fn_or_name = styles[[i]]
                }
            } else if (length(styles) >= i) {
                style_fn_or_name = styles[[i]]
            }

            if (!is.null(style_fn_or_name)) {
                if (is.function(style_fn_or_name)) {
                    context = list(
                        value = value,
                        formatted_value = value,
                        col_name = col_name_or_index,
                        col_index = i,
                        is_header = is_header,
                        data = col_data,
                        justify = "center",
                        width = col_widths[i]
                    )

                    result = tryCatch({
                        styled_result = style_fn_or_name(context)
                        if (is.character(styled_result) && length(styled_result) == 1) {
                            styled_row[i] = styled_result
                        } else {
                            styled_row[i] = value
                        }
                    }, error = function(e) {
                        warning("Styling function failed for column '", col_name_or_index, "': ", e$message, call. = FALSE)
                        value
                    })
                } else if (is.character(style_fn_or_name)) {
                    style_parts = unlist(strsplit(style_fn_or_name, "_"))
                    styled_text = value

                    for (style_part in style_parts) {
                        styled_text = tryCatch({
                            if (exists(paste0("col_", style_part), where = asNamespace("cli"))) {
                                style_fn = get(paste0("col_", style_part), envir = asNamespace("cli"))
                                style_fn(styled_text)
                            } else if (exists(paste0("style_", style_part), where = asNamespace("cli"))) {
                                style_fn = get(paste0("style_", style_part), envir = asNamespace("cli"))
                                style_fn(styled_text)
                            } else if (exists(style_part, where = asNamespace("cli"))) {
                                style_fn = get(style_part, envir = asNamespace("cli"))
                                style_fn(styled_text)
                            } else {
                                warning("Unknown cli style '", style_part, "' in '", style_fn_or_name, "'.", call. = FALSE)
                                styled_text
                            }
                        }, error = function(e) {
                            warning("Applying cli style '", style_part, "' failed: ", e$message, call. = FALSE)
                            value
                        })
                    }
                    styled_row[i] = styled_text
                }
            } else {
                styled_row[i] = value
            }
        } else {
            styled_row[i] = value
        }
    }

    justify_values = character(length(row))
    for (i in seq_along(row)) {
        col_name_or_index = original_names[i]
        justify_value = "center"

        if (!is.null(justify_cols)) {
            temp_justify = NULL
            if (is.list(justify_cols)) {
                if (!is.null(names(justify_cols))) {
                    if (col_name_or_index %in% names(justify_cols)) {
                        temp_justify = justify_cols[[col_name_or_index]]
                    } else if (as.character(i) %in% names(justify_cols)) {
                        temp_justify = justify_cols[[as.character(i)]]
                    }
                }
                if (is.null(temp_justify) && is.null(names(justify_cols)) && i <= length(justify_cols)) {
                    temp_justify = justify_cols[[i]]
                }
            } else if (is.character(justify_cols)) {
                temp_justify = justify_cols[min(i, length(justify_cols))]
            }
            if (!is.null(temp_justify) && temp_justify %in% c("left", "right", "center")) {
                justify_value = temp_justify
            }
        }
        justify_values[i] = justify_value
    }

    displayed_widths = numeric(length(row))
    for (i in seq_along(row)) {
        plain_text = gsub("\033\\[[0-9;]*m", "", styled_row[i])
        displayed_widths[i] = max(nchar(plain_text), nchar(raw_content[i]))
    }

    formatted = character(length(row))
    for (i in seq_along(row)) {
        value = raw_content[i]
        target_width = max(col_widths[i], displayed_widths[i])
        plain_text = gsub("\033\\[[0-9;]*m", "", styled_row[i])
        justified_text = justify_text(plain_text, target_width, justify_values[i])
        if (!grepl("\033\\[", styled_row[i])) {
            formatted[i] = justified_text
            next
        }
        ansi_codes = gregexpr("\033\\[[0-9;]*m", styled_row[i])
        if (length(ansi_codes[[1]]) > 0 && ansi_codes[[1]][1] != -1) {
            code_positions = ansi_codes[[1]]
            ansi_codes_text = regmatches(styled_row[i], ansi_codes)[[1]]

            total_codes_len = sum(nchar(ansi_codes_text))
            text_len = nchar(styled_row[i]) - total_codes_len

            padding = target_width - text_len

            if (justify_values[i] == "left") {
                padding_right = padding
                padding_left = 0
            } else if (justify_values[i] == "right") {
                padding_right = 0
                padding_left = padding
            } else {
                padding_left = floor(padding / 2)
                padding_right = padding - padding_left
            }

            left_pad = paste0(rep(" ", padding_left), collapse = "")
            right_pad = paste0(rep(" ", padding_right), collapse = "")

            first_code_pos = code_positions[1]
            if (first_code_pos == 1) {
                result = paste0(
                    ansi_codes_text[1],
                    left_pad,
                    substr(
                        styled_row[i], nchar(ansi_codes_text[1]) + 1, nchar(styled_row[i])
                    ),
                    right_pad
                )
            } else {
                result = paste0(left_pad, styled_row[i], right_pad)
            }

            formatted[i] = result
        } else {
            formatted[i] = justified_text
        }
    }

    # ---- Vertical Borders ----
    border_char = vb$char %||% getOption("tab_default")$vb_char
    after_cols_spec = vb$after

    if (length(after_cols_spec) > 0) {
        after_cols_idx = integer(0)
        if (is.numeric(after_cols_spec)) {
            after_cols_idx = after_cols_spec
        } else if (is.character(after_cols_spec)) {
            after_cols_idx = match(after_cols_spec, original_names)
        }

        valid_indices = !is.na(after_cols_idx) & after_cols_idx > 0 & after_cols_idx < length(formatted)
        after_cols_idx = sort(unique(after_cols_idx[valid_indices]))

        if (length(after_cols_idx) > 0) {
            for (i in after_cols_idx) {
                if (grepl("\033\\[", formatted[i])) {
                    last_code_pos = max(gregexpr("\033\\[[0-9;]*m", formatted[i])[[1]])
                    if (last_code_pos != -1) {
                        last_code = regmatches(formatted[i], regexpr("\033\\[[0-9;]*m$", formatted[i]))
                        if (length(last_code) > 0 && last_code == "\033[0m") {
                            formatted[i] = sub("\033\\[0m$", paste0(" ", border_char, "\033[0m"), formatted[i])
                        } else {
                            formatted[i] = paste0(formatted[i], " ", border_char)
                        }
                    } else {
                        formatted[i] = paste0(formatted[i], " ", border_char)
                    }
                } else {
                    formatted[i] = paste0(formatted[i], " ", border_char)
                }
            }
        }
    }

    # ---- Combine Cells with Separators ----
    separator = paste0(rep(" ", n_space), collapse = "")
    paste0("  ", paste(formatted, collapse = separator), "  ")
}

#' Generate and Display a Cross Tabulation Table
#'
#' @param data A matrix or table of observed frequencies.
#' @param percentage Which percentages to show. Options: `TRUE`/`"all"`, `"by_row"`,
#'   `"by_col"`, `"by_total"`, `FALSE`/`NULL`.
#' @param layout Show layout legend box? Default `TRUE`.
#' @param expected Show expected frequencies? Default `TRUE`.
#' @param layout_center Center the layout box? Default `FALSE`.
#' @param header_underline Shorten the underline beneath the column header? Default `FALSE`.
#' @param digits Named list with keys `ex`, `row_pct`, `col_pct`, `total_pct`.
#'   Falls back to `getOption("ct_digits")`.
#' @param center_table Center entire table in terminal? Default `FALSE`.
#' @param style Named list supplied using [ct_style()].
#' @param ... Reserved for future use.
#'
#' @return Invisibly returns the formatted cross-tabulation matrix.
#'
#' @examples
#' cross_table(matrix(c(10, 20, 30, 40), nrow = 2))
#' cross_table(matrix(c(10, 20, 30, 40), nrow = 2), percentage = "all")
#'
#' @export
cross_table = function(
        data,
        percentage = NULL,
        layout = TRUE,
        expected = TRUE,
        layout_center = FALSE,
        header_underline = FALSE,
        digits = NULL,
        center_table = FALSE,
        style = NULL,
        ...
) {
    # ---- Validate style ----
    if (!is.null(style) && !is.list(style))
        cli::cli_abort("{.arg style} must be a list or a style object (e.g. {.fn ct_style}).")

    if (inherits(style, "tabstats_style") && !inherits(style, "ct_style")) {
        cli::cli_abort(
            "{.arg style} must be an {.cls ct_style} object for {.fn table_summary}.",
            "x" = "Got {.cls {class(style)[1]}}."
        )
    }

    # ---- Data ----
    observed = as.matrix(data)
    row_totals = rowSums(observed)
    col_totals = colSums(observed)
    grand_total = sum(observed)
    expected_vals = outer(row_totals, col_totals) / grand_total

    digits = digits_resolver(digits)
    pct_flags = pct_resolver(percentage)
    style = style_resolver(style)

    row_pcts = sweep(observed, 1, row_totals, "/") * 100
    col_pcts = sweep(observed, 2, col_totals, "/") * 100
    grand_pcts = observed / grand_total * 100

    # ---- Metadata: To display ----
    row_names = if (is.null(rownames(observed))) paste0("Row", seq_len(nrow(observed))) else rownames(observed)
    col_names = if (is.null(colnames(observed))) paste0("Col", seq_len(ncol(observed))) else colnames(observed)
    main_row = if (is.null(names(dimnames(observed))[1])) "x" else names(dimnames(observed))[1]
    main_col = if (is.null(names(dimnames(observed))[2])) "y" else names(dimnames(observed))[2]

    border_char =
        if (is.character(style$border) && nchar(style$border) == 1) {
            style$border
        } else {
            options("tab_default")$tab_default$border_char
        }

    # ---- Build cell matrix ----
    nr = nrow(observed)
    nc = ncol(observed)
    combined = matrix("", nrow = nr + 1, ncol = nc + 1)

    for (i in seq_len(nr)) {
        for (j in seq_len(nc)) {
            combined[i, j] = format_cell_ct(
                observed[i, j], expected_vals[i, j],
                row_pcts[i, j], col_pcts[i, j], grand_pcts[i, j],
                FALSE, expected, pct_flags, digits, style
            )
        }
        combined[i, nc + 1] = format_cell_ct(
            row_totals[i], row_totals[i],
            100, NA, row_totals[i] / grand_total * 100,
            TRUE, expected, pct_flags, digits, style
        )
    }
    for (j in seq_len(nc)) {
        combined[nr + 1, j] = format_cell_ct(
            col_totals[j], col_totals[j],
            NA, 100, col_totals[j] / grand_total * 100,
            TRUE, expected, pct_flags, digits, style
        )
    }
    grand_pct_val = if (is.null(percentage) || length(percentage) == 0) NA else 100
    combined[nr + 1, nc + 1] = format_cell_ct(
        grand_total, grand_total, NA, NA, grand_pct_val,
        TRUE, expected, pct_flags, digits, style
    )

    all_row_names = c(row_names, "TOTAL")
    all_col_names = c(main_row, col_names, "TOTAL")
    combined = cbind(all_row_names, combined)

    # ---- Column widths ----
    col_widths = vapply(seq_len(ncol(combined)), function(j) {
        max(
            nchar(strip_ansi(all_col_names[j])),
            vapply(seq_len(nrow(combined)), function(i) {
                cell_width_ct(combined[i, j])
            }, integer(1))
        )
    }, integer(1))

    total_width = sum(col_widths) + 3 * (ncol(combined) - 1) + 4
    main_col_width = sum(col_widths[-1]) + 3 * (length(col_widths) - 2)

    hline_raw = strrep(border_char, total_width)
    hline = styler_ct(style$border_text, hline_raw, "border")

    # ---- Layout box -----
    if (layout) {
        print_ct_layout(expected, pct_flags, style, layout_center, center_table)
    }

    # ---- Table lines ----
    lines = line_builder_ct(
        combined, all_col_names, col_widths,
        main_col, main_col_width, total_width,
        header_underline, center_table, hline, style,
        border_char, pct_flags
    )

    pad = if (center_table) {
        term_w = tryCatch(
            cli::console_width(),
            error = function(e) as.integer(getOption("width"))
        )
        strrep(" ", max(0L, floor((term_w - nchar(strip_ansi(lines[[1]]))) / 2L)))
    } else {
        ""
    }

    for (line in lines) cat(pad, line, "\n", sep = "")

    invisible(combined)
}

---
title: "Getting Started"
output:
    rmarkdown::html_vignette:
        toc: true
vignette: >
  %\VignetteIndexEntry{Getting Started}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
    collapse = FALSE,
    comment = NULL
)

options(cli.unicode = FALSE)
options(crayon.enabled = TRUE)

ansi_aware_handler = function(x, options) {
    paste0(
        "<pre class=\"r-output\"><code>",
        fansi::sgr_to_html(x = x, warn = FALSE, term.cap = "256"),
        "</code></pre>"
    )
}

knitr::knit_hooks$set(
    output = ansi_aware_handler, 
    message = ansi_aware_handler, 
    warning = ansi_aware_handler,
    error = ansi_aware_handler
)

knitr::opts_chunk$set(
    collapse = TRUE,
    # comment = "#>",
    comment = NA, 
    # fig.path = "man/figures/README-",
    out.width = "100%"
)

library(tabstats)
```

## Rationale

`{tabstats}` is a lightweight package for displaying data in clean, formatted tables directly in the console. It is mainly designed for developers who wants to display their outputs (preferably in a data frame object), and to make the output readable at a glance, configurable without boilerplate, and friendly to interactive workflows.

The package provides four main functions:

| Function          | Purpose                                    |
|-------------------|--------------------------------------------|
| `table_default()` | General-purpose data frame table           | 
| `table_summary()` | Two-column key-value summary table         |
| `corr_matrix()`   | Correlation matrix display                 |
| `cross_table()`   | Cross tabulation with optional percentages |

All functions share a common design philosophy:

1.  **Styling** is controlled via a `style = list(...)` argument that accepts either predefined style strings (e.g. `"blue_bold"`) or plain lambda functions (e.g. `\(x) cli::col_red(x)`).
2.  **Alignment** is controlled via an `align` argument that accepts a single string, a length-2 vector, or a named list.
3.  **Centering** is controlled via `center_table = TRUE`, which automatically pads the output to the middle of your terminal.

## Installation

Install the following package from CRAN:

```{r, eval = FALSE}
install.packages("tabstats")
```

Or install the development version from GitHub:

```{r eval = FALSE}
# install.packages("pak")
pak::pak("joshuamarie/tabstats")
## devtools::install_github("joshuamarie/tabstats") 
```

## A Quick Tour

The main API of this package displays the data you wanted to display into a specific format

### `table_default()`

Data frames usually looks like this when displayed:

```{r}
head(mtcars[, 1:5], 5)
```

With `table_default()`, the data frame is now displayed in an APA-style format, with configurable formatting.

```{r}
table_default(head(mtcars[, 1:5], 5))
```

### `table_summary()`

Another form of `table_default()`, except it is best suitable for displaying summarized data, where it requires 2 columns data frame. The simplest use case is a named summary of values, like model diagnostics or descriptive statistics, where the 1st column is the name of the parameter (statistic) while the 2nd column is the estimate.

```{r}
df = data.frame(
    Statistic = c("N", "Mean", "SD", "Min", "Max"),
    Value = c("100", "3.45", "1.20", "1.00", "6.00")
)

table_summary(
    df, 
    title = "Descriptive Statistics", 
    header = TRUE
)
```

*The `header` means you still wanted to display the column names from the original `df` data frame, otherwise, no column names are present. *

### `corr_matrix()`

A much complex but niche function to display the matrix, specifically designed for correlation matrices. If the `display` is a correlation matrix, typically the output of `cor()`, you can directly pass it. 

```{r}
corr_matrix(cor(mtcars[, 1:4]), method = "Pearson")
```

But, if the data you wanted to display is not a matrix, but on another form, you're going to have to configure it by building a custom spec with `new_corr_data()` for full control over which values to appear.

A wild example, assuming you want to display the output from `rstatix::cor_test()`: 

```{r}
cor_mat = 
    iris |> 
    rstatix::cor_test(Sepal.Width, Sepal.Length, Petal.Length) |> 
    dplyr::mutate(
        var1,
        var2,
        cor = format(cor, digits = 2),
        statistic = format(statistic, digits = 2),
        conf_int = paste0(
            "[",
            format(conf.low, digits = 2), 
            ", ",
            format(conf.high, digits = 2),
            "]"
        ),
        
        .keep = "unused"
    ) 

cor_mat |> 
    with({
        corr_matrix(
            new_corr_data(
                var1 = var1,
                var2 = var2,
                corr = cor,
                statistic = statistic,
                pval = p,
                conf_int = conf_int
            ),
            title = "Pearson Correlation Matrix"
        )
    })
```

### `cross_table()`

Another nice function to display the matrices — contingency tables, to be exact. It has (almost) everything to display the contingency table, including the observed frequencies, expected values, and percentages side by side.

```{r}
m = matrix(
    c(10, 20, 30, 40), 
    nrow = 2,
    dimnames = list(
        c("A", "B"), 
        c("X", "Y")
    )
)

cross_table(m, percentage = "all")
```

## Styling Primer

All table functions provides an API to style the table. 

1.  `table_default()` has 2 parameters: `style_colnames` to style the column names you wanted to design, and `style_columns` if you want to apply the style to the entire column you choose to style. Use `td_style()` to configure the style of the column you choose to style. 

2.  `table_summary()` has a `style` parameter to configure the style of the specific column you wanted to design. Use `sm_style()` to configure the style of the specific column. 

3.  `cross_table()` has a `style` parameter to configure the style of the data displayed by the function. Use `ct_style()` to configure the style of the displayed values. 

4.  `corr_matrix()` has a `style` parameter to configure the style of the values you entered, typically from the `display` argument using `new_corr_data()`. Use `cm_style()` to configure those values you assigned into `new_corr_data()` into the displayed table.

The quickest way to style output is with a named string:

```{r}
table_summary(
    df, 
    title = "Descriptive Statistics", 
    header = TRUE, 
    style = sm_style(
        left_col = "blue_bold",
        right_col = "green",
        title = "bold", 
        sep = ":  "
    )
)
```

For full control, pass a lambda instead — it receives the text as its argument:

```{r}
table_summary(
    df, 
    title = "Descriptive Statistics", 
    header = TRUE, 
    style = sm_style(
        left_col = \(x) cli::col_red(cli::style_bold(x)),
        right_col = \(x) cli::col_cyan(x), 
        title = "bold", 
        sep = ":  "
    )
)
```

Hex codes are also supported:

```{r}
table_summary(
    df, 
    title = "Descriptive Statistics", 
    header = TRUE, 
    style = sm_style(
        left_col = function(x) {
            red = cli::make_ansi_style("#BD4444")
            red(cli::style_bold(x))
        },
        right_col = \(x) {
            bluish_color = cli::make_ansi_style("#67A2C5")
            bluish_color(x)
        }, 
        title = "bold", 
        sep = ":  "
    )
)
```

Supported named style strings:

| String                                   | Effect                              |
|------------------------------------------|-------------------------------------|
| `"bold"`                                 | Bold text                           |
| `"italic"`                               | Italic text                         |
| `"blue"`, `"red"`, `"green"`, `"yellow"` | Foreground colour                   |
| `"blue_bold"`                            | Colour + bold (combinable with `_`) |
| `"red_italic"`                           | Colour + italic                     |

More example: Imagine you want to apply for the p-value of the output from `rstatix::cor_test()`, an earlier example. In `corr_matrix()`, you can even conditionally format the specified value from `new_corr_data()`. 

```{r}
cor_mat |> 
    with({
        corr_matrix(
            new_corr_data(
                var1 = var1,
                var2 = var2,
                corr = cor,
                statistic = statistic,
                pval = p,
                conf_int = conf_int
            ),
            title = "Pearson Correlation Matrix",
            style = cm_style(
                pval = function(x) {
                    x_num = as.numeric(x)
                    if (is.na(x_num) || x_num > 0.05) {
                        cli::style_italic(x) 
                    } else if (x_num > 0.01) {
                        reddish = cli::make_ansi_style("#CE2626")
                        reddish(x)
                        # or just cli::col_red(x)
                    } else {
                        cli::style_bold("<0.001")
                    }
                }
            )
        )
    })
```

---
title: "Getting Started"
output:
    rmarkdown::html_vignette:
        toc: true
vignette: >
  %\VignetteIndexEntry{Getting Started}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
    collapse = FALSE,
    comment = NULL
)

options(cli.unicode = FALSE)
options(crayon.enabled = TRUE)

ansi_aware_handler = function(x, options) {
    paste0(
        "<pre class=\"r-output\"><code>",
        fansi::sgr_to_html(x = x, warn = FALSE, term.cap = "256"),
        "</code></pre>"
    )
}

knitr::knit_hooks$set(
    output = ansi_aware_handler, 
    message = ansi_aware_handler, 
    warning = ansi_aware_handler,
    error = ansi_aware_handler
)

knitr::opts_chunk$set(
    collapse = TRUE,
    # comment = "#>",
    comment = NA, 
    # fig.path = "man/figures/README-",
    out.width = "100%"
)

library(tabstats)
```

## Rationale

`{tabstats}` is a lightweight package for displaying data in clean, formatted tables directly in the console. It is mainly designed for developers who wants to display their outputs (preferably in a data frame object), and to make the output readable at a glance, configurable without boilerplate, and friendly to interactive workflows.

The package provides four main functions:

| Function          | Purpose                                    |
|-------------------|--------------------------------------------|
| `table_default()` | General-purpose data frame table           | 
| `table_summary()` | Two-column key-value summary table         |
| `corr_matrix()`   | Correlation matrix display                 |
| `cross_table()`   | Cross tabulation with optional percentages |

All functions share a common design philosophy:

1.  **Styling** is controlled via a `style = list(...)` argument that accepts either predefined style strings (e.g. `"blue_bold"`) or plain lambda functions (e.g. `\(x) cli::col_red(x)`).
2.  **Alignment** is controlled via an `align` argument that accepts a single string, a length-2 vector, or a named list.
3.  **Centering** is controlled via `center_table = TRUE`, which automatically pads the output to the middle of your terminal.

## Installation

Install the following package from CRAN:

```{r, eval = FALSE}
install.packages("tabstats")
```

Or install the development version from GitHub:

```{r eval = FALSE}
# install.packages("pak")
pak::pak("joshuamarie/tabstats")
## devtools::install_github("joshuamarie/tabstats") 
```

## A Quick Tour

The main API of this package displays the data you wanted to display into a specific format

### `table_default()`

Data frames usually looks like this when displayed:

```{r}
head(mtcars[, 1:5], 5)
```

With `table_default()`, the data frame is now displayed in an APA-style format, with configurable formatting.

```{r}
table_default(head(mtcars[, 1:5], 5))
```

### `table_summary()`

Another form of `table_default()`, except it is best suitable for displaying summarized data, where it requires 2 columns data frame. The simplest use case is a named summary of values, like model diagnostics or descriptive statistics, where the 1st column is the name of the parameter (statistic) while the 2nd column is the estimate.

```{r}
df = data.frame(
    Statistic = c("N", "Mean", "SD", "Min", "Max"),
    Value = c("100", "3.45", "1.20", "1.00", "6.00")
)

table_summary(
    df, 
    title = "Descriptive Statistics", 
    header = TRUE
)
```

*The `header` means you still wanted to display the column names from the original `df` data frame, otherwise, no column names are present. *

### `corr_matrix()`

A much complex but niche function to display the matrix, specifically designed for correlation matrices. If the `display` is a correlation matrix, typically the output of `cor()`, you can directly pass it. 

```{r}
corr_matrix(cor(mtcars[, 1:4]), method = "Pearson")
```

But, if the data you wanted to display is not a matrix, but on another form, you're going to have to configure it by building a custom spec with `new_corr_data()` for full control over which values to appear.

A wild example, assuming you want to display the output from `rstatix::cor_test()`: 

```{r}
cor_mat = 
    iris |> 
    rstatix::cor_test(Sepal.Width, Sepal.Length, Petal.Length) |> 
    dplyr::mutate(
        var1,
        var2,
        cor = format(cor, digits = 2),
        statistic = format(statistic, digits = 2),
        conf_int = paste0(
            "[",
            format(conf.low, digits = 2), 
            ", ",
            format(conf.high, digits = 2),
            "]"
        ),
        
        .keep = "unused"
    ) 

cor_mat |> 
    with({
        corr_matrix(
            new_corr_data(
                var1 = var1,
                var2 = var2,
                corr = cor,
                statistic = statistic,
                pval = p,
                conf_int = conf_int
            ),
            title = "Pearson Correlation Matrix"
        )
    })
```

### `cross_table()`

Another nice function to display the matrices — contingency tables, to be exact. It has (almost) everything to display the contingency table, including the observed frequencies, expected values, and percentages side by side.

```{r}
m = matrix(
    c(10, 20, 30, 40), 
    nrow = 2,
    dimnames = list(
        c("A", "B"), 
        c("X", "Y")
    )
)

cross_table(m, percentage = "all")
```

## Styling Primer

All table functions provides an API to style the table. 

1.  `table_default()` has 2 parameters: `style_colnames` to style the column names you wanted to design, and `style_columns` if you want to apply the style to the entire column you choose to style. Use `td_style()` to configure the style of the column you choose to style. 

2.  `table_summary()` has a `style` parameter to configure the style of the specific column you wanted to design. Use `sm_style()` to configure the style of the specific column. 

3.  `cross_table()` has a `style` parameter to configure the style of the data displayed by the function. Use `ct_style()` to configure the style of the displayed values. 

4.  `corr_matrix()` has a `style` parameter to configure the style of the values you entered, typically from the `display` argument using `new_corr_data()`. Use `cm_style()` to configure those values you assigned into `new_corr_data()` into the displayed table.

The quickest way to style output is with a named string:

```{r}
table_summary(
    df, 
    title = "Descriptive Statistics", 
    header = TRUE, 
    style = sm_style(
        left_col = "blue_bold",
        right_col = "green",
        title = "bold", 
        sep = ":  "
    )
)
```

For full control, pass a lambda instead — it receives the text as its argument:

```{r}
table_summary(
    df, 
    title = "Descriptive Statistics", 
    header = TRUE, 
    style = sm_style(
        left_col = \(x) cli::col_red(cli::style_bold(x)),
        right_col = \(x) cli::col_cyan(x), 
        title = "bold", 
        sep = ":  "
    )
)
```

Supported named style strings:

| String                                   | Effect                              |
|------------------------------------------|-------------------------------------|
| `"bold"`                                 | Bold text                           |
| `"italic"`                               | Italic text                         |
| `"blue"`, `"red"`, `"green"`, `"yellow"` | Foreground colour                   |
| `"blue_bold"`                            | Colour + bold (combinable with `_`) |
| `"red_italic"`                           | Colour + italic                     |

More example: Imagine you want to apply for the p-value of the output from `rstatix::cor_test()`, an earlier example. In `corr_matrix()`, you can even conditionally format the specified value from `new_corr_data()`. 

```{r}
cor_mat |> 
    with({
        corr_matrix(
            new_corr_data(
                var1 = var1,
                var2 = var2,
                corr = cor,
                statistic = statistic,
                pval = p,
                conf_int = conf_int
            ),
            title = "Pearson Correlation Matrix",
            style = cm_style(
                pval = function(x) {
                    x_num = as.numeric(x)
                    if (is.na(x_num) || x_num > 0.05) {
                        cli::style_italic(x) 
                    } else if (x_num > 0.01) {
                        cli::col_red(x)
                    } else {
                        cli::style_bold("<0.001")
                    }
                }
            )
        )
    })
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/style-helpers.R
\name{sm_style}
\alias{sm_style}
\title{Style specification for table_summary()}
\usage{
sm_style(
  left_col = NULL,
  right_col = NULL,
  border_text = NULL,
  title = NULL,
  sep = NULL
)
}
\arguments{
\item{left_col}{Style for the left column. A string (e.g. \code{"blue_bold"})
or a function \verb{\\(x) ...}.}

\item{right_col}{Style for the right column.}

\item{border_text}{Style for the horizontal border lines.}

\item{title}{Style for the title text.}

\item{sep}{A single character used as the column separator (e.g. \code{"|"}).}
}
\value{
An object of class \code{c("sm_style", "tabstats_style")}.
}
\description{
Constructs a validated style object for use with \code{\link[=table_summary]{table_summary()}}.
}
\examples{
sm_style(left_col = "blue_bold", right_col = "green", title = "bold")
sm_style(left_col = \(x) cli::col_red(x), sep = "|")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/style-helpers.R
\name{cm_style}
\alias{cm_style}
\title{Style specification for corr_matrix()}
\usage{
cm_style(...)
}
\arguments{
\item{...}{Named style entries. Names should match the extra field names
passed to \code{\link[=new_corr_data]{new_corr_data()}} (e.g. \code{rho}, \code{pval}, \code{bf}), or the reserved
keys \code{title} and \code{border_text}.}
}
\value{
An object of class \code{c("cm_style", "tabstats_style")}.
}
\description{
Constructs a validated style object for use with \code{\link[=corr_matrix]{corr_matrix()}}.
}
\examples{
cm_style(rho = "blue_bold", pval = "red", title = "bold")
cm_style(rho = \(x) cli::col_cyan(x))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table-pairwise_matrix.R
\name{pairwise_matrix}
\alias{pairwise_matrix}
\alias{corr_matrix}
\title{Display a Pairwise Matrix Table in the Console}
\usage{
pairwise_matrix(
  display,
  title = NULL,
  name = "Pairwise Matrix",
  diag_1 = TRUE,
  digits = 3,
  layout_view = FALSE,
  layout_center = FALSE,
  center_table = FALSE,
  border_char = getOption("tab_default")$border_char,
  style = list(),
  ...
)

corr_matrix(
  display,
  title = NULL,
  diag_1 = TRUE,
  digits = 3,
  layout_view = FALSE,
  layout_center = FALSE,
  center_table = FALSE,
  border_char = getOption("tab_default")$border_char,
  style = list(),
  ...
)
}
\arguments{
\item{display}{A \code{pairwise_spec} object from \code{new_pairwise_data()}, or a
plain symmetric matrix (e.g. from \code{cor()}).}

\item{title}{Label shown above the table. Auto-detected from a \code{title}
attribute on the spec if present; falls back to \code{name}.}

\item{name}{Short name used in the layout legend box header. Default
\code{"Pairwise Matrix"}.}

\item{diag_1}{If \code{TRUE}, diagonal cells always show \code{"1"}. Default \code{TRUE}.}

\item{digits}{Decimal places for numeric formatting. Default \code{3}.}

\item{layout_view}{Show a layout legend box above the table? Default \code{FALSE}.}

\item{layout_center}{Center the layout box in the terminal? Default \code{FALSE}.}

\item{center_table}{Center table in terminal? Default \code{FALSE}.}

\item{border_char}{Border character. Default from \code{getOption("tab_default")}.}

\item{style}{A style list. Keys match the extra field names passed to
\code{new_pairwise_data()} (e.g. \code{rho}, \code{pval}, \code{bf}), plus \code{title} and
\code{border_text}.}

\item{...}{Reserved for future use.}
}
\value{
Invisibly returns the rendered character matrix.
}
\description{
Display a Pairwise Matrix Table in the Console
}
\examples{
# From a plain correlation matrix
pairwise_matrix(cor(mtcars[, 1:4]), title = "Pearson Correlation Matrix")

# Customizable example
spec = new_pairwise_data(
    var1 = c("a", "a", "b"),
    var2 = c("b", "c", "c"),
    rho = c("0.89", "0.79", "0.66"),
    pval = c("<0.001", "<0.001", "<0.001")
)
pairwise_matrix(spec, title = "Pearson Correlation Matrix", layout_view = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/style-helpers.R
\name{ct_style}
\alias{ct_style}
\title{Style specification for cross_table()}
\usage{
ct_style(
  observed = NULL,
  expected = NULL,
  row_percentage = NULL,
  col_percentage = NULL,
  total_percentage = NULL,
  total = NULL,
  title = NULL,
  border_text = NULL
)
}
\arguments{
\item{observed}{Style for observed frequency cells.}

\item{expected}{Style for expected frequency values.}

\item{row_percentage}{Style for row percentage values.}

\item{col_percentage}{Style for column percentage values.}

\item{total_percentage}{Style for grand total percentage values.}

\item{total}{Style for total row/column cells.}

\item{title}{Style for the table title.}

\item{border_text}{Style for the horizontal border lines.}
}
\value{
An object of class \code{c("ct_style", "tabstats_style")}.
}
\description{
Constructs a validated style object for use with \code{\link[=cross_table]{cross_table()}}.
}
\examples{
ct_style(observed = "blue", expected = "yellow", title = "bold")
ct_style(observed = \(ctx) cli::col_green(ctx$formatted_text))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table-cross_table.R
\name{cross_table}
\alias{cross_table}
\title{Generate and Display a Cross Tabulation Table}
\usage{
cross_table(
  data,
  percentage = NULL,
  layout = TRUE,
  expected = TRUE,
  layout_center = FALSE,
  header_underline = FALSE,
  digits = NULL,
  center_table = FALSE,
  style = NULL,
  ...
)
}
\arguments{
\item{data}{A matrix or table of observed frequencies.}

\item{percentage}{Which percentages to show. Options: \code{TRUE}/\code{"all"}, \code{"by_row"},
\code{"by_col"}, \code{"by_total"}, \code{FALSE}/\code{NULL}.}

\item{layout}{Show layout legend box? Default \code{TRUE}.}

\item{expected}{Show expected frequencies? Default \code{TRUE}.}

\item{layout_center}{Center the layout box? Default \code{FALSE}.}

\item{header_underline}{Shorten the underline beneath the column header? Default \code{FALSE}.}

\item{digits}{Named list with keys \code{ex}, \code{row_pct}, \code{col_pct}, \code{total_pct}.
Falls back to \code{getOption("ct_digits")}.}

\item{center_table}{Center entire table in terminal? Default \code{FALSE}.}

\item{style}{Named list supplied using \code{\link[=ct_style]{ct_style()}}.}

\item{...}{Reserved for future use.}
}
\value{
Invisibly returns the formatted cross-tabulation matrix.
}
\description{
Generate and Display a Cross Tabulation Table
}
\examples{
cross_table(matrix(c(10, 20, 30, 40), nrow = 2))
cross_table(matrix(c(10, 20, 30, 40), nrow = 2), percentage = "all")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tabstats-package.R
\docType{package}
\name{tabstats}
\alias{tabstats-package}
\alias{tabstats}
\title{tabstats: A Lightweight Toolkit for Displaying Customizable Tables}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

A lightweight toolkit that provides functions for printing tables from input data in the R console or terminal with customizable formatting. Supported outputs include American Psychological Association (APA)-style tables (American Psychological Association, 2020, ISBN:9781433832178), correlation matrices, contingency tables, and two-column summary tables.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/joshuamarie/tabstats}
  \item Report bugs at \url{https://github.com/joshuamarie/tabstats/issues}
}

}
\author{
\strong{Maintainer}: Joshua Marie \email{joshua.marie.k@gmail.com}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/options.R
\name{tabstats_options}
\alias{tabstats_options}
\title{Manage package options}
\usage{
tabstats_options(category = NULL, option = NULL, value = NULL)
}
\arguments{
\item{category}{A character string specifying the option category (e.g., "tab_default", "tab_digits").
If omitted, returns all option categories.}

\item{option}{A character string specifying the option to retrieve or modify within the category.
For backward compatibility, you can also use a specific option name directly as the \code{category} parameter.}

\item{value}{The new value to assign to the specified option. If NULL, the function returns the current value.}
}
\value{
If no arguments are provided, returns all option categories and their values.
If only \code{category} is provided, returns all options in that category.
If \code{category} and \code{option} are provided without \code{value}, returns the current value of that option.
If all parameters are provided, updates the option and returns the updated option category list invisibly.
}
\description{
This function allows retrieving or modifying the package options across different categories.
If called without arguments, it returns all option categories and their values.
If \code{category} is provided alone, it returns all options in that category.
If \code{category} and \code{option} are provided, it returns the specific option value.
If all three parameters are provided, it updates the specified option.
}
\examples{
# Get all options across all categories
tabstats_options()

# Get all options in the "tab_default" category
tabstats_options("tab_default")

# Get all options in the "tab_digits" category
tabstats_options("tab_digits")

# Get a specific option
tabstats_options("tab_default", "vb_top")
tabstats_options("tab_digits", "ex")

# Using backward compatibility (system will find the right category)
tabstats_options("vb_top")
tabstats_options("ex")

# Modify an option
tabstats_options("tab_default", "border_char", "+")
tabstats_options("tab_digits", "ex", 2)

# Using backward compatibility for modification
tabstats_options("border_char", "+")
tabstats_options("ex", 2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table_summary.R
\name{table_summary}
\alias{table_summary}
\title{Summarize and Display a Two-Column Data Frame as a Formatted Table}
\usage{
table_summary(
  data,
  title = NULL,
  l = NULL,
  header = FALSE,
  center_table = FALSE,
  border_char = "-",
  style = list(),
  align = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame with exactly two columns. The data to be summarized and displayed.}

\item{title}{A character string. An optional title to be displayed above the table.}

\item{l}{An integer. The number of rows to include in the left part of a split table.
If \code{NULL}, the table is not split.}

\item{header}{A logical value. If \code{TRUE}, the column names of \code{data} are displayed as a header.}

\item{center_table}{A logical value. If \code{TRUE}, the table is centered in the terminal.}

\item{border_char}{Character used for borders. Default is \code{"\\u2500"}.}

\item{style}{A list controlling the visual styling of table elements using ANSI formatting.
Can include the following components:
\itemize{
\item \code{left_col}: Styling for the left column values.
\item \code{right_col}: Styling for the right column values.
\item \code{border_text}: Styling for the border.
\item \code{title}: Styling for the title.
\item \code{sep}: Separator character between left and right column.
}

Each style component can be either a predefined style string (e.g., "blue", "red_italic", "bold")
or a function that takes a context list with/without a \code{value} element and returns the styled text.}

\item{align}{Controls the alignment of column values.
Can be specified in three ways:
\itemize{
\item A single string: affects only the left column (e.g., "left", "center", "right").
\item A vector of two strings: affects both columns in order (e.g., c("left", "right")).
\item A list with named components: explicitly specifies alignment for each column
}}

\item{...}{Additional arguments (currently unused).}
}
\value{
This function does not return a value. It prints the formatted table to the console.
}
\description{
This function takes a two-column data frame and formats it into a summary-like table.
The table can be optionally split into two parts, centered, and given a title.
It is useful for displaying summary information in a clean, tabular format.
The function also supports styling with ANSI colors and text formatting through
the \code{{cli}} package and column alignment options.
}
\examples{
# Create a sample data frame
df = data.frame(
    Category = c("A", "B", "C", "D", "E"),
    Value = c(10, 20, 30, 40, 50)
)

# Display the table with a title and header
table_summary(df, title = "Sample Table", header = TRUE)

# Split the table after the second row and center it
table_summary(df, l = 2, center_table = TRUE)

# Use styling and alignment
table_summary(
    df,
    header = TRUE,
    style = list(
        left_col = "blue_bold",
        right_col = "red",
        title = "green",
        border_text = "yellow"
    ),
    align = c("center", "right")
)

# Use custom styling with lambda functions
table_summary(
    df,
    header = TRUE,
    style = sm_style(
        left_col = \(ctx) cli::col_red(ctx), # ctx$value is another option
        right_col = \(ctx) cli::col_blue(ctx)
    ),
    align = list(left_col = "left", right_col = "right")
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table_default.R
\name{table_default}
\alias{table_default}
\title{Display a formatted table in the console}
\usage{
table_default(
  x,
  justify_cols = "center",
  digits = 3,
  digits_by_col = NULL,
  scientific = FALSE,
  na_print = "",
  min_width = NULL,
  border_char = options("tab_default")$tab_default$border_char,
  show_row_names = FALSE,
  center_table = FALSE,
  n_space = 2,
  title = NULL,
  style_colnames = NULL,
  style_columns = NULL,
  nrows = getOption("tab_default")$nrows,
  vb = list(),
  auto_wrap = TRUE,
  wrap_threshold = 1,
  ...
)
}
\arguments{
\item{x}{A data frame or tibble.}

\item{justify_cols}{Alignment: a single string, vector, or named list of "left"/"right"/"center".}

\item{digits}{Digits to round numeric columns to. Default \code{3}.}

\item{digits_by_col}{Named list of per-column digit overrides.}

\item{scientific}{Display numerics in scientific notation? Default \code{FALSE}.}

\item{na_print}{String for missing values. Default \code{""}.}

\item{min_width}{Minimum column width. Default \code{NULL}.}

\item{border_char}{Character for borders. Default \code{"\\u2500"}.}

\item{show_row_names}{Show row names? Default \code{FALSE}.}

\item{center_table}{Center table in terminal? Default \code{FALSE}.}

\item{n_space}{Spaces between columns. Default \code{2}.}

\item{title}{Optional title string above the table.
from \code{\link[=td_style]{td_style()}}, or a named list where each name is a column name or
\code{"title"}, and each value is either a cli style string (e.g. \code{"blue_bold"})
or a function \verb{\\(ctx) ...} receiving a context list.}

\item{style_colnames}{Styling for column header cells. A \code{td_style} object
from \code{\link[=td_style]{td_style()}}, or a named list where each name is a column name or
\code{"title"}, and each value is either a cli style string (e.g. \code{"blue_bold"})
or a function \verb{\\(ctx) ...} receiving a context list.}

\item{style_columns}{Styling for data cells. A \code{td_style} object from
\code{\link[=td_style]{td_style()}}, or a named list where each name is a column name or column
index as a string, and each value is a cli style string or a function
\verb{\\(ctx) ...} receiving a context list with elements \code{value},
\code{formatted_value}, \code{col_name}, \code{col_index}, \code{is_header}, \code{data},
\code{justify}, and \code{width}.}

\item{nrows}{Max rows to display before truncation.}

\item{vb}{Vertical border spec: \code{list(char = "\\u2502", after = c(1, 3))}.}

\item{auto_wrap}{Auto-wrap wide tables? Default \code{TRUE}.}

\item{wrap_threshold}{Fraction of console width before wrapping. Default \code{1}.}

\item{...}{Reserved for future use.}
}
\value{
Invisibly returns the input data as a character matrix after
formatting has been applied. The function is called primarily for
its side effect of printing a styled table to the R console.
Returns \code{invisible(NULL)} early if the input has 0 rows and 0 columns,
or if it has 0 columns.
}
\description{
Display a formatted table in the console
}
\examples{
table_default(head(mtcars))
table_default(head(mtcars), style_columns = td_style(mpg = "cyan", cyl = "magenta"))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/new_pairwise_data.R
\name{new_pairwise_data}
\alias{new_pairwise_data}
\alias{new_corr_data}
\title{Build a pairwise matrix display specification}
\usage{
new_pairwise_data(var1, var2, ...)

new_corr_data(var1, var2, ...)
}
\arguments{
\item{var1}{Character vector of first variable names per pair.}

\item{var2}{Character vector of second variable names per pair.}

\item{...}{Named character vectors of equal length to \code{var1}/\code{var2}.
Each becomes one display row inside the cell (e.g. \code{rho}, \code{pval}, \code{bf}).}
}
\value{
An object of class \code{pairwise_spec}.
}
\description{
Constructs a structured spec object consumed by \code{pairwise_matrix()}. Always
requires \code{var1} and \code{var2} — the pair pattern they encode determines which
triangle(s) of the matrix are filled:
}
\details{
\tabular{ll}{
   Pattern \tab Fills \cr
   \code{var1 < var2} \tab Lower triangle only \cr
   \code{var1 <= var2} \tab Lower triangle + diagonal \cr
   \code{var1 > var2} \tab Upper triangle only \cr
   \code{var1 >= var2} \tab Upper triangle + diagonal \cr
   \code{var1 != var2} \tab Both triangles, no diagonal \cr
   \code{var1 == var2} \tab Full matrix (diag forced to 1) \cr
}


All additional named vectors become display rows inside each cell, rendered
in the order they are supplied.
}
\examples{
new_pairwise_data(
    var1 = c("a", "a", "b"),
    var2 = c("b", "c", "c"),
    rho = c("0.89", "0.79", "0.66"),
    pval = c("<0.001", "<0.001", "<0.001")
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/style-helpers.R
\name{td_style}
\alias{td_style}
\title{Style specification for table_default()}
\usage{
td_style(...)
}
\arguments{
\item{...}{Named style entries. Each name must be a column name, a column
index as a string (e.g. \code{"1"}), or \code{"title"}. Each value is a string or
a function \verb{\\(ctx) ...} where \code{ctx} is a context list.}
}
\value{
An object of class \code{c("td_style", "tabstats_style")}.
}
\description{
Constructs a validated style object for use with \code{\link[=table_default]{table_default()}}.
}
\examples{
td_style(mpg = "cyan", cyl = "magenta")
td_style(mpg = \(ctx) cli::col_red(ctx$value), title = "bold")

}
